package ejbs.samecontainer;

import java.rmi.RemoteException;
import java.util.Properties;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.equity.Equity;
import ejbs.equity.EquityHome;
import ejbs.equity.EquityVO;
import ejbs.password.Password;
import ejbs.password.PasswordHome;

public class HelperBean implements SessionBean {
	final static boolean VERBOSE = true;
	private SessionContext ctx;
	private PasswordHome passwordHome;
	private EquityHome equityHome;
	private boolean loggedIn = false;


	public boolean login(String userName, String password) throws RemoteException {
		Password passwd = null;

		try {
			passwd = passwordHome.findByPrimaryKey(userName);
			loggedIn = passwd.isPasswordValid(password);
		} catch (Exception e) {
			throw new RemoteException("Finding Password object failed");
		}

		return loggedIn;
	}

	private static void log(String s) {
		if (VERBOSE)
			System.out.println(s);
	}

	//-----------------[ LISTING 1.1 ]-------------------------------//
	private PasswordHome getPasswordHome() {

		try {
			Context ic = new InitialContext();
			PasswordHome passwordHome = (PasswordHome) ic.lookup("ejbAccess.passwordHome");

			return passwordHome;
		} catch (NamingException ne) {
			return null;
		}
	}
	//	-----------------[ LISTING 1.1 ]-------------------------------//

	//	-----------------[ LISTING 1.2 ]-------------------------------//
	private EquityHome getEquityHome() {

		try {
			// Get an InitialContext
			Properties props = new Properties();
			props.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
			props.put(Context.PROVIDER_URL, "http://localhost:7001");
			Context ic = new InitialContext(props);

			EquityHome equityHome = (EquityHome) ic.lookup("ejbAccess.equityHome");
			equityHome = (EquityHome) PortableRemoteObject.narrow(equityHome, EquityHome.class);

			return equityHome;
		} catch (NamingException ne) {
			return null;
		}
	}

	public EquityVO getEquity(String symbol) throws RemoteException {
		//remote interface of the EJB
		Equity equity = null;

		try{
		  //Use previously acquired home reference 
		  //to find an entity instance
		  equity = equityHome.findByPrimaryKey( symbol );
		}
		catch(Exception e){
		  e.printStackTrace();
		  throw new RemoteException("Error Finding Symbol:"+symbol);
		}

		return equity.getEquity();
	}
	//	-----------------[ LISTING 1.2 ]-------------------------------//


	public void setSessionContext(SessionContext c) {

		ctx = c;

		try {
			Context ic = new InitialContext();
			//passwordHome=(PasswordHome)ic.lookup("ejbAccess.passwordHome");
			//equityHome=(EquityHome)ic.lookup("ejbAccess.equityHome");
			passwordHome = getPasswordHome();
			equityHome = getEquityHome();
		} catch (NamingException ne) {
			throw new EJBException(ne);
		}
	}

	public void ejbCreate() {
	}
	public void ejbActivate() {
	}
	public void ejbPassivate() {
	}
	public void ejbRemove() {
	}

}
