package tests.comparebeans;

import java.rmi.RemoteException;
import java.util.Properties;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.ObjectNotFoundException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.equity.Equity;
import ejbs.equity.EquityHome;

public class Client {
	private String url;
	private EquityHome home;

	public Client(String url) throws NamingException {

		this.url = url;
		home = lookupHome();
	}

	private EquityHome lookupHome() throws NamingException {
		Context ctx = getInitialContext();

		try {
			log("Looking up Home:ejbAccess.equityHome");
			Object home = (EquityHome) ctx.lookup("ejbAccess.equityHome");
			return (EquityHome) PortableRemoteObject.narrow(home, EquityHome.class);

		}
		catch (NamingException ne) {
			log(
				"The client was unable to lookup the EJBHome.  Please make sure "
					+ "that you have deployed the ejb with the JNDI name "
					+ "equityHome on the WebLogic server at "
					+ url);
			throw ne;
		}
	}


	//-------------------------------[ Listing 1.15 ]-----------------------------------------//
	public void runExample() {
		Equity equity1 = null;
		Equity equity2 = null;
		String symbol = "ACME";
		String description = "ACME Corp";

		try {

			equity1 = findEquity(symbol, description);
			equity2 = findEquity(symbol, description);

			System.out.println("Does equity1 equal equity2: " + equity1.isIdentical(equity2));
			symbol = "CDE";
			description = "CDE Corp";
			equity1 = findEquity(symbol, description);

			symbol = "ACME";
			description = "ACME Corp";
			equity2 = findEquity(symbol, description);

			System.out.println("Does equity1 equal equity2: " + equity1.isIdentical(equity2));
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	//-------------------------------[ Listing 1.15 ]-----------------------------------------//

	private Context getInitialContext() throws NamingException {

		try {
			// Get an InitialContext
			Properties props = new Properties();
			props.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
			props.put(Context.PROVIDER_URL, url);
			return new InitialContext(props);
		}
		catch (NamingException ne) {
			log("We were unable to get a connection to the WebLogic server at " + url);
			log("Please make sure that the server is running.");
			throw ne;
		}
	}

	private Equity findEquity(String symbol, String description)
		throws CreateException, RemoteException, FinderException {
		try {
			log("Trying to find equity with symbol: " + symbol);
			return (Equity) PortableRemoteObject.narrow(home.findByPrimaryKey(symbol), Equity.class);
		}
		catch (ObjectNotFoundException onfe) {
			log("Equity Not found .... Creating One with symbol: " + symbol);
			return (Equity) PortableRemoteObject.narrow(home.create(symbol, description), Equity.class);
		}
	}

	private static void log(String s) {
		System.out.println(s);
	}

	public static void main(String[] args) {
		String url = "t3://localhost:7001";
		Client client = null;

		System.out.println("\nBeginning comparebeans.Client...\n");

		try {
			client = new Client(url);
		}
		catch (NamingException ne) {
			log("Unable to look up the beans home: " + ne.getMessage());
			System.exit(1);
		}

		try {
			client.runExample();
		}
		catch (Exception e) {
			log("There was an exception while creating and using the Equity.");
			log("This indicates that there was a problem communicating with the server: " + e);
		}

		System.out.println("\nEnd comparebeans.Client...\n");
	}

}
