package tests.ejbpersistence;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.samecontainer.Helper;
import ejbs.samecontainer.HelperHome;

public class ClientSaver {

	//---------------------------[ Listing 1.12 ]----------------------------------------//
	private HelperHome home;

	public void runExample() {

		try {
			home = retrieveHelperHome();
			Helper goodHelper = null;
			goodHelper = (Helper) PortableRemoteObject.narrow(home.create(), Helper.class);
			saveReference(goodHelper);
		}
		catch (Exception e) {
			log("Error creating Helper: ");
			e.printStackTrace();
			System.exit(1);
		}

	}

	private void saveReference(Helper helper) {
		FileOutputStream ostream = null;
		ObjectOutputStream objStream = null;

		try {
			ostream = new FileOutputStream("Helper.obj");
			objStream = new ObjectOutputStream(ostream);
			objStream.writeObject(helper.getHandle());
			objStream.flush();
			objStream.close();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	//---------------------------[ Listing 1.12 ]----------------------------------------//


	private HelperHome retrieveHelperHome() throws NamingException {
		Context ctx = getInitialContext();

		try {
			log("Looking up Home:ejbAccess.helperHome");
			Object home = (HelperHome) ctx.lookup("ejbAccess.helperHome");
			return (HelperHome) PortableRemoteObject.narrow(home, HelperHome.class);

		}
		catch (NamingException ne) {
			log(
				"The client was unable to lookup the storeStatefulRef.HelperHome.  Please make sure "
					+ "that you have deployed the ejb with the JNDI name "
					+ "ejbAccess.helperHome on the WebLogic server at "
					+ "t3://localhost:7001");
			throw ne;
		}
	}


	private Context getInitialContext() throws NamingException {

		try {
			// Get an InitialContext
			Properties props = new Properties();
			props.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
			props.put(Context.PROVIDER_URL, "t3://localhost:7001");
			return new InitialContext(props);
		}
		catch (NamingException ne) {
			log("We were unable to get a connection to the WebLogic server at " + "t3://localhost:7001");
			log("Please make sure that the server is running.");
			throw ne;
		}
	}

	private static void log(String s) {
		System.out.println(s);
	}

	public static void main(String[] args) {
		ClientSaver client = null;

		System.out.println("\nBeginning storeStatefulRef.ClientSaver...\n");

		client = new ClientSaver();

		try {
			client.runExample();
		}
		catch (Exception e) {
			log("There was an exception while creating and using the Helper.");
			log("This indicates that there was a problem communicating with the server: " + e);
		}

		System.out.println("\nEnd storeStatefulRef.ClientSaver...\n");
	}

}
