package tests.equity;

import java.rmi.RemoteException;
import java.util.Properties;

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.FinderException;
import javax.ejb.ObjectNotFoundException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.equity.Equity;
import ejbs.equity.EquityHome;
import ejbs.equity.EquityVO;

public class Client {
	private String url;
	private String symbol;
	private String description;
	private EquityHome home;

	public Client(String symbol, String description, String url) throws NamingException {

		this.url = url;
		this.symbol = symbol;
		this.description = description;
		home = lookupHome();
	}

	private EquityHome lookupHome() throws NamingException {
		Context ctx = getInitialContext();

		try {
			log("Looking up Home:ejbAccess.equityHome");
			Object home = (EquityHome) ctx.lookup("ejbAccess.equityHome");
			return (EquityHome) PortableRemoteObject.narrow(home, EquityHome.class);

		} catch (NamingException ne) {
			log(
				"The client was unable to lookup the EJBHome.  Please make sure "
					+ "that you have deployed the ejb with the JNDI name "
					+ "ejbAccess.equityHome on the WebLogic server at "
					+ url);
			throw ne;
		}
	}

	public void runExample() {
		EquityVO equityVO = new EquityVO();
		Equity equity = null;

		log("Starting the example...\n");

		try {
			log("Creating or Finding An Equity with symbol " + symbol);
			equity = findOrCreateEquity(symbol, description);
			log("Created or Found An Equity with symbol " + symbol);
			equityVO = equity.getEquity();
			log("Created or Found An Equity:\n" + equityVO);
			equityVO.lastTrade = 3.66;
			equityVO.volume += 100;
			equity.setEquity(equityVO);
			log("Current Equity:\n" + equityVO);
		} catch (DuplicateKeyException dke) {
			dke.printStackTrace();
			log("An Equity with symbol " + symbol + " already exists!");
		} catch (Exception e) {
			log("Error creating Equity: " + e.getMessage());
			System.exit(1);
		}

		log("End Part A of the example...\n");
	}

	private Context getInitialContext() throws NamingException {

		try {
			// Get an InitialContext
			Properties props = new Properties();
			props.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
			props.put(Context.PROVIDER_URL, url);
			return new InitialContext(props);
		} catch (NamingException ne) {
			log("We were unable to get a connection to the WebLogic server at " + url);
			log("Please make sure that the server is running.");
			throw ne;
		}
	}

	private Equity findOrCreateEquity(String symbol, String description)
		throws CreateException, RemoteException, FinderException {
		try {
			log("Trying to find equity with symbol: " + symbol);
			return (Equity) PortableRemoteObject.narrow(home.findByPrimaryKey(symbol), Equity.class);
		} catch (ObjectNotFoundException onfe) {
			log("Equity Not found .... Creating One with symbol: " + symbol);
			return (Equity) PortableRemoteObject.narrow(home.create(symbol, description), Equity.class);
		}
	}

	private static void log(String s) {
		System.out.println(s);
	}

	public static void main(String[] args) {
		String url = "t3://localhost:7001";
		String symbol = "ITWO";
		String description = "i2 Technologies";
		Client client = null;

		System.out.println("\nBeginning tests.equity.Client...\n");

		switch (args.length) {
			case 3 :
				symbol = args[1];
				description = args[2];
			case 1 :
				url = args[0];
				break;
		}

		try {
			client = new Client(symbol, description, url);
		} catch (NamingException ne) {
			log("Unable to look up the beans home: " + ne.getMessage());
			System.exit(1);
		}

		try {
			client.runExample();
		} catch (Exception e) {
			log("There was an exception while creating and using the Equity.");
			log("This indicates that there was a problem communicating with the server: " + e);
		}

		System.out.println("\nEnd Equity.Client...\n");
	}

}
