package tests.localservletaccess;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import ejbs.login.Login;
import ejbs.login.LoginHome;

public class LoginServlet extends HttpServlet {

	//	-----------------[ LISTING 1.3 ]-------------------------------//
	//home interface for the EJB PasswordBean
	private LoginHome loginHome = null;

	public void init(ServletConfig conf) throws ServletException {
		super.init(conf);
		try {
			lookupLoginHome();
		}
		catch (NamingException e) {
			e.printStackTrace();
		}
	}

	public void doGet(HttpServletRequest req, HttpServletResponse res) {
		String name = getUserName(req);
		String password = getUserPassword(req);

		Login loginBean;
		try {
			lookupLoginHome();
			//Uncomment this for listing 1.3
			//loginBean = loginHome.create(name);

			//	-----------------[ LISTING 1.4 ]-------------------------------//
			loginBean = (Login) PortableRemoteObject.narrow(loginHome.create(name), Login.class);
			//	-----------------[ LISTING 1.4 ]-------------------------------//

			Boolean valid = loginBean.login(password);
			sendReplyToClient(req, res, "Login Status:" + valid);
		}
		catch (Exception e) {
			e.printStackTrace();
			String msg = buildException(e);
			sendErrReply(res, msg);
		}

		//perform further work (not shown)
	}

	public void doPost(HttpServletRequest req, HttpServletResponse res) {
		doGet(req, res);
	}

	private void lookupLoginHome() throws NamingException {
		//	-----------------[ LISTING 1.4 ]-------------------------------//
		Properties props = new Properties();

		props.put(Context.INITIAL_CONTEXT_FACTORY, getInitParameter("factory_class"));
		props.put(Context.PROVIDER_URL, getInitParameter("url"));

		Context ctx = new InitialContext(props);
		//	-----------------[ LISTING 1.4 ]-------------------------------//

		//Uncomment this for Listing 1.3
		//Context ctx = new InitialContext();
		try {
			if (loginHome == null)
				loginHome = (LoginHome) ctx.lookup("servletAccess.loginHome");
		}
		catch (NamingException ne) {
			throw ne;
		}
	}
	//	-----------------[ LISTING 1.3 ]-------------------------------//

	private String getUserName(HttpServletRequest req) {
		String userName = req.getParameter("Username");

		return userName;
	}

	private String getUserPassword(HttpServletRequest req) {
		String passwd = req.getParameter("Password");

		return passwd;
	}

	protected void sendErrReply(HttpServletResponse resp, String str) {
		StringBuffer buffer = new StringBuffer();

		try {
			ServletOutputStream outputStream = resp.getOutputStream();

			buffer.append("<HTML>\n");
			buffer.append("<BODY>\n");
			buffer.append("<PRE>\n");
			buffer.append("<H1>\n");
			buffer.append("REPLY\n");
			buffer.append("</H1>\n");
			buffer.append("<HR>\n");

			buffer.append(str);

			buffer.append("<HR>\n");
			buffer.append("</PRE>\n");
			buffer.append("</BODY>\n");
			buffer.append("</HTML>\n");

			log("------------[ Sending Reply ]-------------");
			log(buffer.toString());
			log("------------[ Sending Reply ]-------------");

			resp.sendError(200, buffer.toString());
		}
		catch (Exception e) {
			log("Error Sending Reply message to User:", e);
			e.printStackTrace();
		}

	}

	public String getServletInfo() {
		return "Client Access to EJB.";
	}

	public String buildException(Exception e) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		PrintWriter writer = new PrintWriter(baos);
		StringBuffer msg = new StringBuffer();

		e.printStackTrace(writer);
		writer.flush();
		writer.close();
		msg.append(e.getMessage());
		msg.append("\n");
		msg.append("-----------[ Stack Trace ]----------------------\n");
		msg.append(new String(baos.toByteArray()));
		msg.append("\n");
		msg.append("-----------[ Stack Trace ]----------------------\n");

		return msg.toString();
	}

	public void log(String str, Exception e) {
		System.out.println("ClientServlet::" + str);
	}

	protected void sendReplyToClient(HttpServletRequest req, HttpServletResponse resp, String str) throws IOException {
		ServletOutputStream outputStream = resp.getOutputStream();
		StringBuffer buffer = new StringBuffer();
		String reply = null;

		buffer.append("<HTML>\n");
		buffer.append("<BODY>\n");
		buffer.append("<PRE>\n");
		buffer.append("<H1>\n");
		buffer.append("REPLY\n");
		buffer.append("</H1>\n");
		buffer.append("<HR>\n");

		buffer.append(str);

		buffer.append("<HR>\n");
		buffer.append("</PRE>\n");
		buffer.append("</BODY>\n");
		buffer.append("</HTML>\n");

		reply = buffer.toString();

		log("\n" + "\n------------[ Sending ]-------------\n" + reply + "\n------------[ Sending ]-------------\n");

		resp.setContentType("text/html");
		outputStream.write(reply.getBytes(), 0, reply.length());
		outputStream.flush();
	}

}
