package tests.reflection;

import java.lang.reflect.Method;
import java.util.Properties;

import javax.ejb.EJBHome;
import javax.ejb.EJBMetaData;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import ejbs.equity.Equity;

public class Client {
	private String url;
	private String symbol;
	private String description;
	private EJBHome home;

	//------------------------[ Listing 1.16 ]--------------------------------//
	public void runExample() {
		Equity equity = null;
		Context ctx = null;

		try {
			ctx = getInitialContext();
			home = (EJBHome) ctx.lookup("ejbAccess.equityHome");
			home = (EJBHome) javax.rmi.PortableRemoteObject.narrow(home, EJBHome.class);

			EJBMetaData metaData = home.getEJBMetaData();

			Class homeClass = metaData.getHomeInterfaceClass();
			Class remoteClass = metaData.getRemoteInterfaceClass();

			System.out.println("---------[ MetaData Info ]-----------");
			System.out.println("Home Class Type:" + homeClass.getName());
			System.out.println("Remote Class Type:" + remoteClass.getName());

			home = (EJBHome) javax.rmi.PortableRemoteObject.narrow(home, homeClass);

			System.out.println("Primary Key  Type:" + metaData.getPrimaryKeyClass().getName());
			System.out.println("Is this a Session " + "Bean:" + metaData.isSession());
			System.out.println("Is this a Stateless Session Bean:" + metaData.isStatelessSession());

			Method[] methods = homeClass.getDeclaredMethods();
			Method[] remoteMethods = remoteClass.getDeclaredMethods();

			System.out.println("---------[ Home Methods ]-----------");
			for (int i = 0; i < methods.length; i++) {
				System.out.println("Method Found:" + methods[i].getName());
				Class[] params = remoteMethods[i].getParameterTypes();
				System.out.println("  Has " + params.length + " Parameters");
				for (int j = 0; j < params.length; j++) {
					System.out.println("    Param:" + params[j].getName());
				}
			}
			System.out.println("---------[ Home Methods ]--------\n\n");

			System.out.println("---------[ Remote Methods ]-------");
			for (int i = 0; i < remoteMethods.length; i++) {
				System.out.println("\nMethod " + "Found:" + remoteMethods[i].getName());
				Class[] params = remoteMethods[i].getParameterTypes();
				for (int j = 0; j < params.length; j++) {
					System.out.println("      Param:" + params[j].getName());
				}
			}
			System.out.println("-----[ Remote Methods ]--------\n\n");
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		System.out.println("End of the example...\n");
	}
	//------------------------[ Listing 1.16 ]--------------------------------//

	public Client(String url) throws NamingException {

		this.url = url;
	}


	private Context getInitialContext() throws NamingException {

		try {
			// Get an InitialContext
			Properties props = new Properties();
			props.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
			props.put(Context.PROVIDER_URL, url);
			return new InitialContext(props);
		}
		catch (NamingException ne) {
			System.out.println("We were unable to get a connection to the WebLogic server at " + url);
			System.out.println("Please make sure that the server is running.");
			throw ne;
		}
	}


	public static void main(String[] args) {
		String url = "t3://localhost:7001";
		Client client = null;

		System.out.println("\nBeginning tests.reflection.Client...\n");


		try {
			client = new Client(url);
		}
		catch (NamingException ne) {
			System.out.println("Unable to look up the beans home: " + ne.getMessage());
			System.exit(1);
		}

		try {
			client.runExample();
		}
		catch (Exception e) {
			System.out.println("There was an exception while creating and using the Equity.");
			System.out.println("This indicates that there was a problem communicating with the server: " + e);
		}

		System.out.println("\nEnd Equity.Client...\n");
	}

}
