package tests.samecontainer;

import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.equity.EquityVO;
import ejbs.samecontainer.Helper;
import ejbs.samecontainer.HelperHome;

public class Client {
	private String url;
	private HelperHome home;

	public Client(String url) throws NamingException {

		this.url = url;
		home = lookupHome();
	}

	private HelperHome lookupHome() throws NamingException {
		Context ctx = getInitialContext();

		try {
			log("Looking up Home:ejbAccess.helperHome");
			Object home = (HelperHome) ctx.lookup("ejbAccess.helperHome");
			return (HelperHome) PortableRemoteObject.narrow(home, HelperHome.class);

		} catch (NamingException ne) {
			log(
				"The client was unable to lookup the ejbAccess.helperHome.  Please make sure "
					+ "that you have deployed the ejb with the JNDI name "
					+ "ejbAccess.helperHome on the WebLogic server at "
					+ url);
			throw ne;
		}
	}

	public void runExample() {
		Helper goodHelper = null;
		Helper badHelper = null;

		log("Starting the example...\n");

		try {
			//-----------------[ Creating a good Helper ]-------------//
			log("Creating A Good Helper");
			goodHelper = (Helper) PortableRemoteObject.narrow(home.create(), Helper.class);
			boolean isLoggedIn = goodHelper.login("markw", "passwd");
			log("User: markw with Password: passwd logged in OK:" + isLoggedIn);

			//-----------------[ Creating a bad Helper ]-------------//
			log("Creating A Bad Helper");
			badHelper = (Helper) PortableRemoteObject.narrow(home.create(), Helper.class);
			isLoggedIn = badHelper.login("markw", "badpasswd");
			log("User: markw with Password: badpasswd logged in OK:" + isLoggedIn);

			log("---------[ Good Helper ]------------");
			EquityVO equity = goodHelper.getEquity("ITWO");
			log(equity.toString());
			log("---------[ Good Helper ]------------");

			log("---------[ bad Helper ]------------");
			equity = badHelper.getEquity("ITWO");
			log(equity.toString());
			log("---------[ bad Helper ]------------");

		} catch (Exception e) {
			log("Error creating Helper: ");
			e.printStackTrace();
			System.exit(1);
		}

		log("End of the example...\n");
	}

	private Context getInitialContext() throws NamingException {

		try {
			// Get an InitialContext
			Properties props = new Properties();
			props.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
			props.put(Context.PROVIDER_URL, url);
			return new InitialContext(props);
		} catch (NamingException ne) {
			log("We were unable to get a connection to the WebLogic server at " + url);
			log("Please make sure that the server is running.");
			throw ne;
		}
	}

	private static void log(String s) {
		System.out.println(s);
	}

	public static void main(String[] args) {
		String url = "t3://localhost:7001";
		Client client = null;

		System.out.println("\nBeginning EJBAccess.Client...\n");

		try {
			client = new Client(url);
		} catch (NamingException ne) {
			log("Unable to look up the beans home: " + ne.getMessage());
			System.exit(1);
		}

		try {
			client.runExample();
		} catch (Exception e) {
			log("There was an exception while creating and using the Helper.");
			log("This indicates that there was a problem communicating with the server: " + e);
		}

		System.out.println("\nEnd EJBAccess.Client...\n");
	}

}
