package ejbs.BMPreadOnly;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.RemoveException;
import javax.naming.InitialContext;
import javax.sql.DataSource;


public class PasswordBean implements EntityBean {
  private EntityContext             ctx;
  public String                     userName=null;
  public String                     password=null;

  public void setEntityContext(EntityContext ctx) {
    this.ctx = ctx;
  }

  public void unsetEntityContext() {
    this.ctx = null;
  }

  public void ejbActivate(){
      log("PasswordBean.ejbActivate (" + userName + ")");
  }

  public void ejbPassivate(){
    log("PasswordBean.ejbPassivate (" + userName + ")");
  }


  public void ejbLoad(){
    PreparedStatement          ps = null;
    Connection                 con=null;

    log("PasswordBean.ejbLoad (" + userName +  ")");

    try {
      String query="select password from PASSWORD where owner_name = ?";

      log("Executing Query:"+query);

      con=getConnection();

      ps  = con.prepareStatement(query);
      ps.setString(1, userName);
      ps.executeQuery();
      ResultSet rs = ps.getResultSet();
      if (rs.next()) {
        password = (rs.getString(1)).trim();
        log("ejbLoad::FindByPrimaryKeyLocated Password:"+userName);
      }
      else {
        String error = "ejbLoad: PasswordBean (" + userName + ") not found";
        log(error);
        throw new ObjectNotFoundException (error);
      }
    }
    catch (Exception sqe) {
      log("SQLException:  " + sqe);
      throw new EJBException (sqe);
    }
    finally {
      try{
        if(ps!=null) ps.close();
        if(con!=null) con.close();
      }
      catch(SQLException e){}
    }

  }

  public void ejbStore() {
    log("PasswordBean.ejbStore (" + userName + ") Not Storing Object");
  }

  public void ejbRemove()throws RemoveException{
    log("PasswordBean.ejbRemove (" + userName + ") Method not allowed");

    throw new RemoveException("Remove Method Not allowed on this bean");
  }

  public boolean isValid(String password){
    log("Checking db password:"+this.password+" against password:"+password);
    return (this.password.equals(password));
  }

  public String ejbFindByPrimaryKey(String pk) throws ObjectNotFoundException{
    PreparedStatement          ps = null;
    Connection                 con=null;

    log("ejbFindByPrimaryKey (" + pk + ")");

    try {
      String query="select owner_name from PASSWORD where owner_name = ?";
      log("Executing Query:"+query);


      con=getConnection();
      ps  = con.prepareStatement(query);
      ps.setString(1, pk);
      ps.executeQuery();
      ResultSet rs = ps.getResultSet();
      if (rs.next()) {
        userName = rs.getString(1);
        log("FindByPrimaryKeyLocated Password:"+userName);
      }
      else {
        String error = "ejbFindByPrimaryKey: PasswordBean (" + pk + ") not found";
        log(error);
        throw new ObjectNotFoundException (error);
      }
    }
    catch (SQLException sqe) {
      log("SQLException:  " + sqe);
      throw new EJBException (sqe);
    } finally {
      try{
        if(ps!=null) ps.close();
        if(con!=null) con.close();
      }
      catch(SQLException e){}
    }

    log("ejbFindByPrimaryKey (" + pk + ") found");
    return pk;
  }

  public Connection getConnection() {
    DataSource            ds=null;

    try{
      ds = (javax.sql.DataSource) (new InitialContext()).lookup("java:comp/env/ejbBookDataSource");
      return ds.getConnection();
    }
    catch(Exception e){
      e.printStackTrace();
    }

    return null;
  }

  private void log(Object s) {
    System.out.println(s);
  }

}
