package ejbs.BMPsequence;
import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.NoSuchEntityException;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.RemoveException;
import javax.naming.InitialContext;
import javax.sql.DataSource;

public class SequenceBean implements EntityBean {	
	private EntityContext ctx;
	private int sequenceId = 0;
	private String name;

	//---------------------------------[ Listing 3.7 ]-----------------------------------------------------	
	public Integer ejbCreate( String name ) throws CreateException{
	  PreparedStatement ps = null;
	  Connection        con = null;
	  ResultSet         rs = null;

	  this.name = name;

	  try {
		String query = "select test_sequence.nextval from dual";
		con = getConnection();
		ps  = con.prepareStatement( query );
		ps.executeQuery();

		rs = ps.getResultSet();

		if( rs.next() )
		{
		  sequenceId = rs.getInt(1);
		}
		else {
		  String error = "ejbCreate: Sequence error creating";
		  System.out.println(error);
		  throw new CreateException (error);
		}

		ps.close();
		rs.close();

		query = "insert into ejbSequence( sequenceId, name ) values (?,?)";
		ps = con.prepareStatement(query);
		ps.setInt(1, sequenceId); 
		ps.setString(2, name);
		if ( !(ps.executeUpdate() > 0) ) {
		  String error = "ejbCreate: SequenceBean (" 
							 + name + ") not created";
		  System.out.println(error);
		  throw new NoSuchEntityException (error);
		}

		ps.close();

	  }
	  catch(SQLException sqe) {
		throw new EJBException (sqe);
	  }
	  finally {
		try{
		  if(ps!=null) ps.close();
		  if(rs!=null) rs.close();
		  if(con!=null) con.close();
		}
		catch(SQLException e){}
	  }

	  return new Integer( sequenceId );
	}

    //---------------------------------[ Listing 3.7 ]-----------------------------------------------------
	

	public void setEntityContext(EntityContext ctx) {
		this.ctx = ctx;
	}

	public void unsetEntityContext() {
		this.ctx = null;
	}

	public void ejbActivate() {
		log("SequenceBean.ejbActivate (" + sequenceId + ")");
	}

	public void ejbPassivate() {
		log("SequenceBean.ejbPassivate (" + sequenceId + ")");
	}

	public void ejbPostCreate(String name) {
		log("SequenceBean.ejbPostCreate (" + name + ")");
	}

	public void ejbLoad() {
		PreparedStatement ps = null;
		Connection con = null;

		log("SequenceBean.ejbLoad (" + name + ")");

		try {
			String query = "select sequenceId,name from ejbSequence where sequenceId = ?";

			log("Executing Query:" + query);

			con = getConnection();

			ps = con.prepareStatement(query);
			ps.setInt(1, sequenceId);
			ps.executeQuery();
			ResultSet rs = ps.getResultSet();
			if (rs.next()) {
				sequenceId = rs.getInt(1);
				name = rs.getString(2);
				log("ejbLoad::FindByPrimaryKeyLocated Sequence:" + name);
			}
			else {
				String error = "ejbLoad: SequenceBean (" + name + ") not found";
				log(error);
				throw new ObjectNotFoundException(error);
			}

		}
		catch (Exception sqe) {
			log("SQLException:  " + sqe);
			throw new EJBException(sqe);
		}
		finally {
			try {
				if (ps != null)
					ps.close();
				if (con != null)
					con.close();
			}
			catch (SQLException e) {
			}
		}

	}

	public void ejbStore() {
		log("SequenceBean.ejbStore (" + name + ")");

		PreparedStatement ps = null;
		Connection con = null;

		try {
			String query = "update ejbSequence set " + "name = ? " + "where sequenceId = ?";
			log("Executing Query:" + query);
			con = getConnection();

			ps = con.prepareStatement(query);
			ps.setString(1, name);
			ps.setInt(2, sequenceId);
			if (!(ps.executeUpdate() > 0)) {
				String error = "ejbStore: SequenceBean (" + name + ") not updated";
				log(error);
				throw new NoSuchEntityException(error);
			}
		}
		catch (SQLException sqe) {
			log("SQLException:  " + sqe);
			throw new EJBException(sqe);
		}
		finally {
			try {
				if (ps != null)
					ps.close();
				if (con != null)
					con.close();
			}
			catch (SQLException e) {
			}
		}

	}

	public void ejbRemove() throws RemoveException {
		PreparedStatement ps = null;
		Connection con = null;

		log("SequenceBean.ejbRemove (" + name + ")");

		try {
			String query = "delete from ejbSequence where sequenceId = ?";
			log("Executing Query:" + query);

			con = getConnection();
			ps = con.prepareStatement(query);
			ps.setInt(1, sequenceId);
			if (!(ps.executeUpdate() > 0)) {
				String error = "ejbRemove: SequenceBean (" + name + ") not Removed";
				log(error);
				throw new NoSuchEntityException(error);
			}
		}
		catch (SQLException sqe) {
			log("SQLException:  " + sqe);
			throw new EJBException(sqe);
		}
		finally {
			try {
				if (ps != null)
					ps.close();
				if (con != null)
					con.close();
			}
			catch (SQLException e) {
			}
		}

	}


	public Integer ejbFindByPrimaryKey(Integer pk) throws ObjectNotFoundException {
		PreparedStatement ps = null;
		Connection con = null;
		ResultSet rs = null;

		log("ejbFindByPrimaryKey (" + pk + ")");

		try {
			String query = "select sequenceId from ejbSequence where sequenceId = ?";
			log("Executing Query:" + query);

			con = getConnection();
			ps = con.prepareStatement(query);
			ps.setInt(1, pk.intValue());
			ps.executeQuery();
			rs = ps.getResultSet();
			if (rs.next()) {
				sequenceId = rs.getInt(1);
				log("FindByPrimaryKeyLocated Sequence:" + sequenceId);
			}
			else {
				String error = "ejbFindByPrimaryKey: SequenceBean (" + pk + ") not found";
				log(error);
				throw new ObjectNotFoundException(error);
			}
		}
		catch (SQLException sqe) {
			log("SQLException:  " + sqe);
			throw new EJBException(sqe);
		}
		finally {
			try {
				if (ps != null)
					ps.close();
				if (rs != null)
					rs.close();
				if (con != null)
					con.close();
			}
			catch (SQLException e) {
			}
		}

		log("ejbFindByPrimaryKey (" + pk + ") found");
		return pk;
	}

	public Connection getConnection() {
		DataSource ds = null;

		try {
			ds = (javax.sql.DataSource) (new InitialContext()).lookup("java:comp/env/ejbBookDataSource");
			return ds.getConnection();
		}
		catch (Exception e) {
			e.printStackTrace();
		}

		return null;
	}

	public int getSequenceNum() throws RemoteException {
		return sequenceId;
	}

	public String getName() throws RemoteException {
		return name;
	}

	private void log(Object s) {
		System.out.println(s);
	}

}
