package ejbs.compoundKey;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.NoSuchEntityException;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.RemoveException;
import javax.naming.InitialContext;
import javax.sql.DataSource;

public class PortfolioHoldingBean implements EntityBean {
	private EntityContext ctx;
	private PortfolioHoldingVO holding = new PortfolioHoldingVO();

	public void setEntityContext(EntityContext ctx) {
		this.ctx = ctx;
	}

	public void unsetEntityContext() {
		this.ctx = null;
	}

	public void ejbActivate() {
		log("PortfolioHoldingBean.ejbActivate (" + holding.portfolioName + ")");
	}

	public void ejbPassivate() {
		log("PortfolioHoldingBean.ejbPassivate (" + holding.portfolioName + ")");
	}

	public void ejbPostCreate(PortfolioHoldingVO holding) {
		log("PortfolioHoldingBean.ejbPostCreate (" + holding + ")");
	}

	public void ejbLoad() {
		PreparedStatement ps = null;
		Connection con = null;
		HoldingKey key = null;

		try {
			key = (HoldingKey) ctx.getPrimaryKey();
			log("PortfolioHoldingBean.ejbLoad (" + key + ")");
			String query =
				"select portfolioName,symbol,quantity,cost from "
					+ "ejbPortfolioHolding where portfolioName=? and symbol=?";

			log("Executing Query:" + query);
			con = getConnection();
			ps = con.prepareStatement(query);
			ps.setString(1, key.portfolioName);
			ps.setString(2, key.symbol);
			ps.executeQuery();
			ResultSet rs = ps.getResultSet();
			if (rs.next()) {
				holding.portfolioName = rs.getString(1);
				holding.symbol = rs.getString(2);
				holding.quantity = rs.getInt(3);
				holding.cost = rs.getDouble(4);
				log("ejbLoad::FindByPrimaryKeyLocated PortfolioHolding:" + holding);
			}
			else {
				String error = "ejbLoad: PortfolioHoldingBean (" + key + ") not found";
				log(error);
				throw new ObjectNotFoundException(error);
			}
		}
		catch (Exception sqe) {
			log("SQLException:  " + sqe);
			throw new EJBException(sqe);
		}
		finally {
			try {
				if (ps != null)
					ps.close();
				if (con != null)
					con.close();
			}
			catch (SQLException e) {
			}
		}

	}

	public void ejbStore() {
		log("PortfolioHoldingBean.ejbStore (" + holding + ")");

		PreparedStatement ps = null;
		Connection con = null;

		try {
			String query = "update ejbPortfolioHolding set quantity=?,cost=? " + "where portfolioName=? and symbol=?";
			log("Executing Query:" + query);
			con = getConnection();
			ps = con.prepareStatement(query);
			ps.setInt(1, holding.quantity);
			ps.setDouble(2, holding.cost);
			ps.setString(3, holding.portfolioName);
			ps.setString(4, holding.symbol);
			if (!(ps.executeUpdate() > 0)) {
				String error = "ejbStore: PortfolioHoldingBean (" + holding + ") not updated";
				log(error);
				throw new NoSuchEntityException(error);
			}
		}
		catch (SQLException sqe) {
			log("SQLException:  " + sqe);
			throw new EJBException(sqe);
		}
		finally {
			try {
				if (ps != null)
					ps.close();
				if (con != null)
					con.close();
			}
			catch (SQLException e) {
			}
		}

	}

	public void ejbRemove() throws RemoveException {
		PreparedStatement ps = null;
		Connection con = null;

		log("PortfolioHoldingBean.ejbRemove (" + holding + ")");

		try {
			String query = "delete from ejbPortfolioHolding where " + "portfolioName=? and symbol=?";
			log("Executing Query:" + query);
			con = getConnection();
			ps = con.prepareStatement(query);
			ps.setString(1, holding.portfolioName);
			ps.setString(2, holding.symbol);
			if (!(ps.executeUpdate() > 0)) {
				String error = "ejbRemove:PortfolioHoldingBean(" + holding + ") not Removed";
				log(error);
				throw new NoSuchEntityException(error);
			}
		}
		catch (SQLException sqe) {
			log("SQLException:  " + sqe);
			throw new EJBException(sqe);
		}
		finally {
			try {
				if (ps != null)
					ps.close();
				if (con != null)
					con.close();
			}
			catch (SQLException e) {
			}
		}

	}

	public HoldingKey ejbCreate(PortfolioHoldingVO holding) throws CreateException {
		PreparedStatement ps = null;
		Connection con = null;
		HoldingKey key = new HoldingKey();

		log("PortfolioHoldingBean.ejbCreate(" + holding + ")");
		this.holding = holding;

		try {
			String query = "insert into ejbPortfolioHolding(portfolioName,symbol,quantity,cost) values(?,?,?,?)";
			log("Executing Query:" + query);
			con = getConnection();
			ps = con.prepareStatement(query);
			ps.setString(1, holding.portfolioName);
			ps.setString(2, holding.symbol);
			ps.setInt(3, holding.quantity);
			ps.setDouble(4, holding.cost);
			if (!(ps.executeUpdate() > 0)) {
				String error = "ejbCreate: PortfolioHoldingBean(" + holding + ") not created";
				log(error);
				throw new NoSuchEntityException(error);
			}
		}
		catch (SQLException sqe) {
			log("SQLException:  " + sqe);
			throw new EJBException(sqe);
		}
		finally {
			try {
				if (ps != null)
					ps.close();
				if (con != null)
					con.close();
			}
			catch (SQLException e) {
			}
		}

		log("EJBCREATE:");

		key.portfolioName = holding.portfolioName;
		key.symbol = holding.symbol;
		return key;
	}

	public HoldingKey ejbFindByPrimaryKey(HoldingKey pk) throws ObjectNotFoundException {
		PreparedStatement ps = null;
		Connection con = null;

		log("ejbFindByPrimaryKey (" + pk + ")");

		try {
			String query =
				"select portfolioName,symbol from ejbPortfolioHolding " + "where portfolioName=? and symbol=?";
			log("Executing Query:" + query);
			con = getConnection();
			ps = con.prepareStatement(query);
			ps.setString(1, pk.portfolioName);
			ps.setString(2, pk.symbol);
			ps.executeQuery();
			ResultSet rs = ps.getResultSet();
			if (rs.next()) {
				pk.portfolioName = rs.getString(1);
				pk.symbol = rs.getString(2);
				log("FindByPrimaryKeyLocated PortfolioHolding:" + pk);
			}
			else {
				String error = "ejbFindByPrimaryKey: PortfolioHoldingBean (" + pk + ") not found";
				log(error);
				throw new ObjectNotFoundException(error);
			}
		}
		catch (SQLException sqe) {
			log("SQLException:  " + sqe);
			throw new EJBException(sqe);
		}
		finally {
			try {
				if (ps != null)
					ps.close();
				if (con != null)
					con.close();
			}
			catch (SQLException e) {
			}
		}

		log("ejbFindByPrimaryKey (" + pk + ") found");
		return pk;
	}


	//-------------------------------[ Listing 3.12 ]----------------------------------------------------
	public Collection ejbFindByPortfolioName( String portfolioName ) 
							  throws ObjectNotFoundException 
	{
	  ArrayList         array = new ArrayList();
	  PreparedStatement ps = null;
	  Connection        con = null;
	  HoldingKey        key = null;

	  try {
      
		String query = "select portfolioName,symbol "  
			   + "from ejbPortfolioHolding where portfolioName=?"; 

		con = getConnection();
		ps  = con.prepareStatement(query);

		ps.setString(1, portfolioName);
		ps.executeQuery();

		ResultSet rs = ps.getResultSet();

		while( rs.next() ) {
		  key = new HoldingKey();
		  key.portfolioName = rs.getString(1);
		  key.symbol = rs.getString(2);
		  array.add(key);
		}
	  }
	  catch (SQLException sqe) {

		throw new EJBException (sqe);
	  }
	  finally {
		try{
		  if(ps!=null) ps.close();
		  if(con!=null) con.close();
		}
		catch(SQLException e){}
	  }

	  return array;
	}
	//-------------------------------[ Listing 3.12 ]----------------------------------------------------

	public void update(PortfolioHoldingVO newHolding) {
		log("=====> updateHolding Method Called:" + newHolding);
		this.holding = newHolding;
	}

	public PortfolioHoldingVO getHolding() {
		log("----------> getPortfolioHolding Method Called");

		return holding;
	}

	public Connection getConnection() {
		DataSource ds = null;

		try {
			ds = (javax.sql.DataSource) (new InitialContext()).lookup("java:comp/env/ejbBookDataSource");
			return ds.getConnection();
		}
		catch (Exception e) {
			e.printStackTrace();
		}

		return null;
	}

	private void log(String s) {
		System.out.println(s);
	}

}
