package ejbs.notifications;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.NoSuchEntityException;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.RemoveException;
import javax.naming.InitialContext;
import javax.sql.DataSource;


public class EquityBean implements EntityBean {
  final static boolean              VERBOSE = true;
  private EntityContext             ctx;
  public String                     symbol=null;
  public String                     description=null;
  public double                     lastTrade=0.0;
  public double                     change=0.0;
  private int                       volume=0;
  public double                     mktCap=0.0;
  public double                     PE=0.0;
  public int                        avgVolume=0;
  private transient boolean         isDirty;   

  public void setEntityContext(EntityContext ctx) {
    this.ctx = ctx;
  }

  public void unsetEntityContext() {
    this.ctx = null;
  }


  public void ejbActivate(){
      log("EquityBean.ejbActivate (" + symbol + ")");
  }

  public void ejbPassivate(){
    log("EquityBean.ejbPassivate (" + symbol + ")");
  }

  public void ejbPostCreate(String symbol, String description){
    log("EquityBean.ejbPostCreate (" + symbol + ")");
  }

  public void ejbLoad(){
    PreparedStatement          ps = null;
    Connection                 con=null;

    log("EquityBean.ejbLoad (" + symbol +  ")");

    try {
      String query="select description, "+
                   "lastTrade, "+
                   "change, "+
                   "volume, "+
                   "marketCap, "+
                   "PE, "+
                   "avgVolume "+
                   "from EQUITIES where symbol = ?";

      log("Executing Query:"+query);

      con=getConnection();

      ps  = con.prepareStatement(query);
      ps.setString(1, symbol);
      ps.executeQuery();
      ResultSet rs = ps.getResultSet();
      if (rs.next()) {
        description = rs.getString(1);
        lastTrade=rs.getFloat(2);
        change=rs.getFloat(3);
        volume=rs.getInt(4);
        mktCap=rs.getInt(5);
        PE=rs.getFloat(6);
        avgVolume=rs.getInt(7);
        log("ejbLoad::FindByPrimaryKeyLocated Equity:"+symbol);
      }
      else {
        String error = "ejbLoad: EquityBean (" + symbol + ") not found";
        log(error);
        throw new ObjectNotFoundException (error);
      }
    }
    catch (Exception sqe) {
      log("SQLException:  " + sqe);
      throw new EJBException (sqe);
    }
    finally {
      try{
        if(ps!=null) ps.close();
        if(con!=null) con.close();
      }
      catch(SQLException e){}
    }

  }

  public void ejbStore() {
    log("EquityBean.ejbStore (" + symbol + ")");

    PreparedStatement         ps = null;
    Connection                con=null;

    try {
      String query="update EQUITIES set "+
                   "description = ?,"+
                   "lastTrade = ?,"+
                   "change = ?,"+
                   "volume = ?,"+
                   "marketcap = ?,"+
                   "PE = ?,"+
                   "avgVolume = ? "+
                   "where symbol = ?";
      log("Executing Query:"+query);
      con=getConnection();

      ps = con.prepareStatement(query);
      ps.setString(1, description);
      ps.setDouble(2,lastTrade);
      ps.setDouble(3,change);
      ps.setInt(4,volume);
      ps.setDouble(5,mktCap);
      ps.setDouble(6,PE);
      ps.setInt(7,avgVolume);
      ps.setString(8,symbol);
      if (!(ps.executeUpdate() > 0)) {
        String error = "ejbStore: EquityBean (" + symbol + ") not updated";
        log(error);
        throw new NoSuchEntityException (error);
      }
    }
    catch(SQLException sqe) {
      log("SQLException:  " + sqe);
      throw new EJBException (sqe);
    }
    finally {
      try{
        if(ps!=null) ps.close();
        if(con!=null) con.close();
      }
      catch(SQLException e){}
    }

  }

  public void ejbRemove()throws RemoveException{
    PreparedStatement           ps = null;
    Connection                  con=null;

    log("EquityBean.ejbRemove (" + symbol + ")");

    try {
      String query="delete from EQUITIES where symbol = ?";
      log("Executing Query:"+query);

      con=getConnection();
      ps = con.prepareStatement(query);
      ps.setString(1, symbol);
      if (!(ps.executeUpdate() > 0)) {
        String error = "ejbRemove: EquityBean (" + symbol + ") not Removed";
        log(error);
        throw new NoSuchEntityException (error);
      }
    }
    catch(SQLException sqe) {
      log("SQLException:  " + sqe);
      throw new EJBException (sqe);
    }
    finally {
      try{
        if(ps!=null) ps.close();
        if(con!=null) con.close();
      }
      catch(SQLException e){}
    }

    publishMessage("Removed Equity from Database:"+symbol);

    //setModified(false);
  }

  public String ejbCreate(String symbol, String description) throws CreateException{
    PreparedStatement      ps = null;
    Connection             con=null;

    log("EquityBean.ejbCreate(PK="+symbol+", "+"Description="+description+")");
    this.symbol=symbol;
    this.description=description;


    try {
      String query="insert into EQUITIES (symbol,description) values(?,?)";
      log("Executing Query:"+query);


      con=getConnection();
      ps = con.prepareStatement(query);
      ps.setString(1, symbol);
      ps.setString(2, description);
      if (!(ps.executeUpdate() > 0)) {
        String error = "ejbCreate: EquityBean (" + symbol + ") not created";
        log(error);
        throw new NoSuchEntityException (error);
      }
    }
    catch(SQLException sqe) {
      log("SQLException:  " + sqe);
      throw new EJBException (sqe);
    }
    finally {
      try{
        if(ps!=null) ps.close();
        if(con!=null) con.close();
      }
      catch(SQLException e){}
    }

    //setModified(false);

    log("EJBCREATE:"+this);

    publishMessage("Inserted Equity into Database:"+symbol);
    return symbol;
  }

  public String ejbFindByPrimaryKey(String pk) throws ObjectNotFoundException{
    PreparedStatement          ps = null;
    Connection                 con=null;

    log("ejbFindByPrimaryKey (" + pk + ")");

    try {
      String query="select symbol from EQUITIES where symbol = ?";
      log("Executing Query:"+query);


      con=getConnection();
      ps  = con.prepareStatement(query);
      ps.setString(1, pk);
      ps.executeQuery();
      ResultSet rs = ps.getResultSet();
      if (rs.next()) {
        symbol = rs.getString(1);
        log("FindByPrimaryKeyLocated Equity:"+symbol);
      }
      else {
        String error = "ejbFindByPrimaryKey: EquityBean (" + pk + ") not found";
        log(error);
        throw new ObjectNotFoundException (error);
      }
    }
    catch (SQLException sqe) {
      log("SQLException:  " + sqe);
      throw new EJBException (sqe);
    } finally {
      try{
        if(ps!=null) ps.close();
        if(con!=null) con.close();
      }
      catch(SQLException e){}
    }

    log("ejbFindByPrimaryKey (" + pk + ") found");
    return pk;
  }

  public void setEquity(EquityVO equity){
    log("----------> setEquity Method Called");
    this.volume=equity.volume;
    this.symbol=equity.symbol;
    this.description=equity.description;
    this.lastTrade=equity.lastTrade;
    this.change=equity.change;
    this.volume=equity.volume;
    this.mktCap=equity.mktCap;
    this.PE=equity.PE;
    this.avgVolume=equity.avgVolume;
    publishMessage("Changed the Equity in the Database:"+symbol);
  }

  public EquityVO getEquity() {
    EquityVO           equity=new EquityVO();

    log("----------> getEquity Method Called");
    equity.symbol=this.symbol;
    equity.volume=this.volume;
    equity.symbol=this.symbol;
    equity.description=this.description;
    equity.lastTrade=this.lastTrade;
    equity.change=this.change;
    equity.volume=this.volume;
    equity.mktCap=this.mktCap;
    equity.PE=this.PE;

    return equity;
  }

  public Connection getConnection() {
    DataSource            ds=null;

    try{
      ds = (javax.sql.DataSource) (new InitialContext()).lookup("java:comp/env/ejbBookDataSource");
      return ds.getConnection();
    }
    catch(Exception e){
      e.printStackTrace();
    }

    return null;
  }

  private void publishMessage(String msg) {
    JMSPublisher          publisher=null;

     try{
       publisher=new JMSPublisher("BookJMSFactory","BookJMSTopic");
   
       System.out.println("Publishing message: "+msg);
       publisher.publish(msg);
       publisher.close();
     }
     catch(Exception e){
       log("Error Publishing Message:"+msg);
       e.printStackTrace();
     }
 
  }

  private void log(Object s) {
    if (VERBOSE) System.out.println(s);
  }

}
