package ejbs.one2onerelation;

import java.rmi.RemoteException;
import java.sql.Date;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

public class HelperBean implements SessionBean {
	final static int numOwners = 5;
	private SessionContext ctx;
	private AddressHome addressHome;
	private OwnerHome ownerHome;

	public void createAddressesAndOwners() throws RemoteException {
		Address address = null;
		ejbs.one2onerelation.Owner owner = null;
		int i = 0;
		String ownerName = null;
		String addr = null;

		log("\nCreate Addresses and Owners:");

		try {
			for (i = 0; i < numOwners; i++) {
				//---------[ Create the Owner ]----------//
				java.sql.Date lastDate = new Date(System.currentTimeMillis());
				owner = createOwner("owner_" + i, lastDate);

				//---------[ Create the Address ]----------//
				ownerName = "owner_" + i;
				addr = "Address_Owner_" + i;
				address = createAddress(addr);
				log("Setting Address in Owner:" + addr);
				owner.setAddress(address);
				log("Setting Owner in Address:" + ownerName);
				address.setOwner(owner);
			}

		}
		catch (Exception e) {
			log("Error Creating Owner-Address Relationships");
			e.printStackTrace();
			throw new EJBException(e);
		}

		log("\nCreated " + numOwners + " Owners and Addresses");
	}

	public void printOwners() throws RemoteException {
		log("\nPrint Owners:");

		String ownerName = "";
		Owner owner = null;

		// print out the owners
		for (int i = 0; i < numOwners; i++) {
			try {
				ownerName = "owner_" + i;
				owner = ownerHome.findByPrimaryKey(ownerName);

				log("Owner: " + owner.getOwnerName() + " has addresses:");
			}
			catch (FinderException fe) {
				log("Owner: " + ownerName + " not found");
			}
		}
	}

	public void removeOwner_CascadeDeleteAddresses(int i) throws RemoteException {
		log("\nCascade Delete:");
		log("Removing owner " + i + " and cascade delete its addresses");

		// Remove owner
		String ownerName = "owner_" + i;

		try {
			Owner owner = ownerHome.findByPrimaryKey(ownerName);
			owner.remove();
			System.out.println(ownerName + " removed");
		}
		catch (Exception e) {
			log("Error removing owner " + ownerName + ": " + e.getMessage());
			throw new EJBException(e);
		}
	}

	public void removeAllCustomers() throws RemoteException {
		String addressName = "";
		String ownerName = "";
		Address address = null;
		Owner owner = null;

		log("\nRemoveAllCustomers...");

		for (int i = 0; i < numOwners; i++) {
			try {
				ownerName = "owner_" + i;
				owner = ownerHome.findByPrimaryKey(ownerName);
				owner.remove();
				System.out.println(ownerName + " removed");
			}
			catch (FinderException e) {
				// ignored
			}
			catch (Exception e) {
				log("Error during RemoveAllCustomers: " + e.getMessage());
				throw new EJBException(e);
			}
		}

		log("Done RemoveAllCustomers...\n\n");
	}

	private Address createAddress(String address) throws CreateException, RemoteException {
		Address p = null;

		log("Creating Address: " + address);
		p = (Address) PortableRemoteObject.narrow(addressHome.create(address), Address.class);
		log("Successfully Created Address: " + address);

		return p;
	}

	private Owner createOwner(String name, Date lastAccess) throws CreateException, RemoteException {
		log("Creating Owner:" + name + "...");

		Owner owner = (Owner) PortableRemoteObject.narrow(ownerHome.create(name, lastAccess), Owner.class);
		log("Owner " + name + " successfully created");

		return owner;
	}

	private static void log(String s) {
		System.out.println(s);
	}

	public void setSessionContext(SessionContext c) {

		ctx = c;

		try {
			Context ic = new InitialContext();

			addressHome = (AddressHome) ic.lookup("one2oneRelation.AddressHome");
			ownerHome = (OwnerHome) ic.lookup("one2oneRelation.OwnerHome");

		}
		catch (NamingException ne) {
			throw new EJBException(ne);
		}
	}

	public void ejbCreate() {
	}
	public void ejbActivate() {
	}
	public void ejbPassivate() {
	}
	public void ejbRemove() {
	}

}
