package ejbs.sessionFacade;
import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

public class HelperBean implements SessionBean {
	final static boolean VERBOSE = true;
	final static int numUsers = 5;
	private SessionContext ctx;
	private PasswordHome passwordHome;

	public void createUserList() throws RemoteException {
		Password password = null;

		log("\nCreating Set of users:");

		for (int i = 0; i < numUsers; i++) {
			try {
				//---------[ Create the Password ]----------//
				password = createPassword("user_" + i, "password_" + i);
			}
			catch (Exception e) {
				log("Error Creating Password");
				throw new EJBException(e);
			}
		}

		log("\nCreated " + numUsers + " Users and Passwords");
	}

	public void removeAll() throws RemoteException {
		Password password = null;

		log("\nRemoving Set of users:");

		for (int i = 0; i < numUsers; i++) {
			try {
				//---------[ Create the Password ]----------//
				password = passwordHome.findByPrimaryKey("user_" + i);
				password.remove();
			}
			catch (Exception e) {
				log("Error Creating Password");
				throw new EJBException(e);
			}
		}

		log("\nCreated " + numUsers + " Users and Passwords");
	}

	private Password createPassword(String userName, String password) throws CreateException, RemoteException {
		Password p = null;

		log("Creating user " + userName + " with a password of " + password);
		p = (Password) PortableRemoteObject.narrow(passwordHome.create(userName, password), Password.class);
		log("Password:" + userName + " successfully created");

		return p;
	}

	public boolean login(String userName, String password) throws RemoteException {
		Password passwd = null;

		try {
			passwd = passwordHome.findByPrimaryKey(userName);
			return passwd.isPasswordValid(password);
		}
		catch (Exception e) {
			throw new RemoteException("Finding Password object failed");
		}

	}

	private static void log(String s) {
		if (VERBOSE)
			System.out.println(s);
	}

	public void setSessionContext(SessionContext c) {

		ctx = c;

		try {
			Context ic = new InitialContext();
			passwordHome = (PasswordHome) ic.lookup("sessionFacade.PasswordHome");
		}
		catch (NamingException ne) {
			throw new EJBException(ne);
		}
	}

	public void ejbCreate() {
	}
	public void ejbActivate() {
	}
	public void ejbPassivate() {
	}
	public void ejbRemove() {
	}

}
