package tests.BMPreadOnly;

import java.rmi.RemoteException;
import java.util.Properties;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.ObjectNotFoundException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.BMPreadOnly.Password;
import ejbs.BMPreadOnly.PasswordHome;


public class Client {
  private String url;
  private String userName;
  private String password;
  private PasswordHome home;

  public Client(String userName, String password,String url)throws NamingException {

    this.url       = url;
    this.userName = userName;
    this.password = password;
    home = lookupHome();
  }

  private PasswordHome lookupHome() throws NamingException{
    Context ctx = getInitialContext();

    try {
      log("Looking up Home:passwordHome");
      Object home = (PasswordHome) ctx.lookup("passwordHome");
      return (PasswordHome) PortableRemoteObject.narrow(home, PasswordHome.class);

    }
    catch (NamingException ne) {
      log("The client was unable to lookup the EJBHome.  Please make sure " +
      "that you have deployed the ejb with the JNDI name " + 
      "passwordHome on the WebLogic server at "+url);
      throw ne;
    }
  }

   public void runExample() {
     Password                       passwd = null;

     log("Starting the example...\n");

     try {
       log("Finding An Password with userName " + userName);
       passwd=findPassword(userName);
       log("Found An Password with userName " + userName);

       log("Is Password Valid:" + passwd.isValid(password));
     }
     catch(Exception e) {
       log("Error Finding Password: " + e.getMessage());
     }

     log("Removing Password...\n");
     try {
       passwd.remove();
     } catch(Exception e) {
       log("Error removing Password: "+e.getMessage());
     }

     log("End Part A of the example...\n");
   }



  private Context getInitialContext() throws NamingException {

    try {
      // Get an InitialContext
      Properties props = new Properties();
      props.put(Context.INITIAL_CONTEXT_FACTORY,"weblogic.jndi.WLInitialContextFactory");
      props.put(Context.PROVIDER_URL, url);
      return new InitialContext(props);
    } catch (NamingException ne) {
      log("We were unable to get a connection to the WebLogic server at "+url);
      log("Please make sure that the server is running.");
      throw ne;
    }
  }

  private Password findPassword(String userName)
            throws CreateException, RemoteException, FinderException {
    try {
      log("Trying to find password with userName: "+userName);
      return (Password)PortableRemoteObject.narrow(home.findByPrimaryKey(userName),Password.class);
    }
    catch (ObjectNotFoundException onfe) {
      log("Password Not found ....: "+userName);
      throw new FinderException("Symbol: "+userName+" not found");
    } 
  }

  private static void log(String s) {
    System.out.println(s);
  }

  public static void main(String[] args) {
    String                        url="t3://localhost:7001";
    String                        userName="markw";
    String                        password="password";
    Client                        client = null;

    System.out.println("\nBeginning password.Client...\n");

    switch(args.length) {
    case 3:
      userName = args[1];
      password = args[2];
    case 1:
      url       = args[0];
      break;
    }

    try {
      client = new Client(userName, password,url);
    }
    catch (NamingException ne) {
      log("Unable to look up the beans home: " + ne.getMessage());
      System.exit(1);
    }

    try {
      client.runExample();
    } catch (Exception e) {
      log("There was an exception while creating and using the Password.");
      log("This indicates that there was a problem communicating with the server: "+e);
    }

    System.out.println("\nEnd Password.Client...\n");
  }

}
