package tests.CMPsequence;

import java.util.Properties;

import javax.ejb.DuplicateKeyException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.CMPsequence.Sequence;
import ejbs.CMPsequence.SequenceHome;

public class Client {
	private String url;
	private String name;
	private SequenceHome home;

	public Client(String name, String url) throws NamingException {

		this.url = url;
		this.name = name;
		home = lookupHome();
	}

	private SequenceHome lookupHome() throws NamingException {
		Context ctx = getInitialContext();

		try {
			log("Looking up Home:CMPsequenceHome");
			Object home = (SequenceHome) ctx.lookup("CMPsequence");
			return (SequenceHome) PortableRemoteObject.narrow(home, SequenceHome.class);

		}
		catch (NamingException ne) {
			log(
				"The client was unable to lookup the SequenceHome.  Please make sure "
					+ "that you have deployed the ejb with the JNDI name "
					+ "sequenceHome on the WebLogic server at "
					+ url);
			throw ne;
		}
	}

	public void runExample() {
		Sequence sequenceKey = null;
		int value = -1;

		log("Starting the example...\n");

		try {
			log("Creating A Sequence with name " + name);
			sequenceKey = (Sequence) PortableRemoteObject.narrow(home.create(name), Sequence.class);
			value = (sequenceKey.getSequenceId()).intValue();
			log("Created A Sequence with name:" + name + " and SequenceId:" + value);
		}
		catch (DuplicateKeyException dke) {
			dke.printStackTrace();
			log("An Sequence with key " + value + " already exists!");
		}
		catch (Exception e) {
			log("Error creating Sequence: " + e.getMessage());
			System.exit(1);
		}

		log("End of the example...\n");
	}

	private Context getInitialContext() throws NamingException {

		try {
			// Get an InitialContext
			Properties props = new Properties();
			props.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
			props.put(Context.PROVIDER_URL, url);
			return new InitialContext(props);
		}
		catch (NamingException ne) {
			log("We were unable to get a connection to the WebLogic server at " + url);
			log("Please make sure that the server is running.");
			throw ne;
		}
	}

	private static void log(String s) {
		System.out.println(s);
	}

	public static void main(String[] args) {
		String url = "t3://localhost:7001";
		String name = "ITWO";
		Client client = null;

		System.out.println("\nBeginning sequence.Client...\n");

		if (args.length > 0)
			name = args[0];

		try {
			client = new Client(name, url);
		}
		catch (NamingException ne) {
			log("Unable to look up the beans home: " + ne.getMessage());
			System.exit(1);
		}

		try {
			client.runExample();
		}
		catch (Exception e) {
			log("There was an exception while creating and using the Sequence.");
			log("This indicates that there was a problem communicating with the server: " + e);
		}

		System.out.println("\nEnd Sequence.Client...\n");
	}

}
