package tests.compoundKey;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Properties;

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.compoundKey.HoldingKey;
import ejbs.compoundKey.PortfolioHolding;
import ejbs.compoundKey.PortfolioHoldingHome;
import ejbs.compoundKey.PortfolioHoldingVO;

public class Client {
	private String url;
	private PortfolioHoldingHome home;
	private PortfolioHoldingVO holding = new PortfolioHoldingVO();
	private HoldingKey key = new HoldingKey();

	public Client(String portfolioName, String symbol, String url) throws NamingException {

		this.url = url;
		holding.portfolioName = portfolioName;
		holding.symbol = symbol;
		holding.quantity = 100;

		home = lookupHome();
	}

	private PortfolioHoldingHome lookupHome() throws NamingException {
		Context ctx = getInitialContext();

		try {
			log("Looking up Home:portfolioHoldingHome");
			Object home = (PortfolioHoldingHome) ctx.lookup("portfolioHoldingHome");
			return (PortfolioHoldingHome) PortableRemoteObject.narrow(home, PortfolioHoldingHome.class);

		}
		catch (NamingException ne) {
			log(
				"The client was unable to lookup the EJBHome.  Please make sure "
					+ "that you have deployed the ejb with the JNDI name "
					+ "portfolioHoldingHome on the WebLogic server at "
					+ url);
			throw ne;
		}
	}

	public void runExample() {
		PortfolioHolding holdingBean = null;

		log("Starting the example...\n");

		try {
			holdingBean = createOrFindPortfolioHolding(holding);
			log("Created An PortfolioHolding:");
		}
		catch (DuplicateKeyException dke) {
			dke.printStackTrace();
			log("An PortfolioHolding " + holding + " already exists!");
		}
		catch (Exception e) {
			log("Error creating PortfolioHolding: " + e.getMessage());
			System.exit(1);
		}

		try {
			holding.quantity += 100;
			holding.cost += 17;
			log("Updating PortfolioHolding:" + holding);
			holdingBean.update(holding);

			ArrayList list = null;

			list = (ArrayList) home.findByPortfolioName("TestPortfolio");
			log("Found: " + list.size() + " portfolioholdings");
			//holdingBean.remove();
		}
		catch (Exception e) {
			e.printStackTrace();
			log("Error removing PortfolioHolding: " + e.getMessage());
		}

		log("End of the example...\n");
	}

	private Context getInitialContext() throws NamingException {

		try {
			// Get an InitialContext
			Properties props = new Properties();
			props.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
			props.put(Context.PROVIDER_URL, url);
			return new InitialContext(props);
		}
		catch (NamingException ne) {
			log("We were unable to get a connection to the WebLogic server at " + url);
			log("Please make sure that the server is running.");
			throw ne;
		}
	}

	private PortfolioHolding createOrFindPortfolioHolding(PortfolioHoldingVO holding)
		throws CreateException, RemoteException {
		HoldingKey key = new HoldingKey();
		PortfolioHolding obj = null;

		log("Creating Or Finding A PortfolioHolding:\n" + holding);

		try {
			key.portfolioName = holding.portfolioName;
			key.symbol = holding.symbol;
			log("Trying to find equity with key:\n" + key);
			obj = (PortfolioHolding) PortableRemoteObject.narrow(home.findByPrimaryKey(key), PortfolioHolding.class);
			this.holding = obj.getHolding();
			log("Holding found .... :\n" + holding);
		}
		catch (Exception onfe) {
			log("Holding Not found .... Creating One with key:\n" + key);
			Object o = home.create(holding);
			return (PortfolioHolding) PortableRemoteObject.narrow(obj, PortfolioHolding.class);
		}

		return obj;
	}

	private static void log(String s) {
		System.out.println(s);
	}

	public static void main(String[] args) {
		String url = "t3://localhost:7001";
		String name = "TestPortfolio";
		String symbol = "ITWO";
		Client client = null;

		System.out.println("\nBeginning portfolioHolding.Client...\n");

		switch (args.length) {
			case 1 :
				symbol = args[0];
				break;
		}

		try {
			client = new Client(name, symbol, url);
		}
		catch (NamingException ne) {
			log("Unable to look up the beans home: " + ne.getMessage());
			System.exit(1);
		}

		try {
			client.runExample();
		}
		catch (Exception e) {
			log("There was an exception while creating and using the PortfolioHolding.");
			log("This indicates that there was a problem communicating with the server: " + e);
		}

		System.out.println("\nEnd PortfolioHolding.Client...\n");
	}

}
