package tests.ebql;

import java.util.*;

import javax.ejb.ObjectNotFoundException;
import javax.rmi.PortableRemoteObject;
import javax.naming.InitialContext;
import javax.ejb.DuplicateKeyException;
import javax.ejb.FinderException;
import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.naming.Context;
import javax.naming.NamingException;

import ejbs.ebql.Equity;
import ejbs.ebql.EquityHome;
import ejbs.ebql.EquityVO;


public class Client {
  private String url;
  private EquityHome home;
  private String                        symbol="ITWO";
  private String                        description="i2 Technologies";

  public Client(String url)throws NamingException {

    this.url       = url;
    home = lookupHome();
  }

  private EquityHome lookupHome() throws NamingException{
    Context ctx = getInitialContext();

    try {
      log("Looking up Home:ebqlEquityHome");
      Object home = (EquityHome) ctx.lookup("ebqlEquityHome");
      return (EquityHome) PortableRemoteObject.narrow(home, EquityHome.class);

    }
    catch (NamingException ne) {
      log("The client was unable to lookup the EJBHome.  Please make sure " +
      "that you have deployed the ejb with the JNDI name " + 
      "ebqlEquityHome on the WebLogic server at "+url);
      throw ne;
    }
  }

   public void runExample() {
     EquityVO                     equityVO=new EquityVO();
     Equity                       equity = null;

     log("Starting the example...\n");

     try {
       log("Creating or Finding An Equity with symbol " + symbol);
       equity=findOrCreateEquity(symbol,description);
       equityVO=equity.getEquity();
       equityVO.lastTrade=3.66;
       equityVO.volume+=100;
       equityVO.PE=70;
       equity.setEquity(equityVO);
       log("Current Equity:\n"+equityVO);

       symbol="SUNW";
       log("Creating or Finding An Equity with symbol " + symbol);
       equity=findOrCreateEquity(symbol,description);
       equityVO=equity.getEquity();
       equityVO.lastTrade=12.00;
       equityVO.volume+=100;
       equityVO.PE=20;
       equity.setEquity(equityVO);
       log("Current Equity:\n"+equityVO);

       symbol="IBM";
       log("Creating or Finding An Equity with symbol " + symbol);
       equity=findOrCreateEquity(symbol,description);
       equityVO=equity.getEquity();
       equityVO.lastTrade=72.00;
       equityVO.volume+=100;
       equityVO.PE=40;
       equity.setEquity(equityVO);
       log("Current Equity:\n"+equityVO);

       findHighPriced(10.0);
       findHighPricedLowPE(10.0,30);

     }
     catch(DuplicateKeyException dke) {
       dke.printStackTrace();
       log("An Equity with symbol " + symbol + " already exists!");
     }
     catch(Exception e) {
       log("Error creating Equity: " + e.getMessage());
       System.exit(1);
     }

     log("End of the example...\n");
   }



  private Context getInitialContext() throws NamingException {

    try {
      // Get an InitialContext
      Properties props = new Properties();
      props.put(Context.INITIAL_CONTEXT_FACTORY,"weblogic.jndi.WLInitialContextFactory");
      props.put(Context.PROVIDER_URL, url);
      return new InitialContext(props);
    } catch (NamingException ne) {
      log("We were unable to get a connection to the WebLogic server at "+url);
      log("Please make sure that the server is running.");
      throw ne;
    }
  }

  private Equity findOrCreateEquity(String symbol, String description)
            throws CreateException, RemoteException, FinderException {
    try {
      log("Trying to find equity with symbol: "+symbol);
      return (Equity)PortableRemoteObject.narrow(home.findByPrimaryKey(symbol),Equity.class);
    }
    catch (ObjectNotFoundException onfe) {
      log("Equity Not found .... Creating One with symbol: "+symbol);
      return (Equity)PortableRemoteObject.narrow(home.create(symbol,description),Equity.class);
    } 
  }


  private void findHighPriced(double minPrice) 
    throws RemoteException, FinderException
  {
    log("\nQuerying for Equities with a price greater than " +minPrice);

    Collection col = home.findHighPriced(minPrice);

    if(col.isEmpty()) {
      log("No Equities were found!");
    }

    Iterator it = col.iterator();
    while (it.hasNext()) {
      Equity equity=(Equity)PortableRemoteObject.narrow(it.next(),Equity.class);

      log("-----------------[ Equity ]--------------------");
      log((equity.getEquity()).toString());
      log("-----------------[ Equity ]--------------------");
    }
  }

  private void findHighPricedLowPE(double minPrice, double PE) 
    throws RemoteException, FinderException
  {
    log("\nQuerying for Equities with a price greater than " +minPrice+
        " and a PE less than "+PE);

    Collection col = home.findHighPricedLowPE(minPrice,PE);

    if(col.isEmpty()) {
      log("No Equities were found!");
    }

    Iterator it = col.iterator();
    while (it.hasNext()) {
      Equity equity=(Equity)PortableRemoteObject.narrow(it.next(),Equity.class);

      log("-----------------[ Equity ]--------------------");
      log((equity.getEquity()).toString());
      log("-----------------[ Equity ]--------------------");
    }
  }



  private static void log(String s) {
    System.out.println(s);
  }

  public static void main(String[] args) {
    String                        url="t3://localhost:7001";
    Client                        client = null;

    System.out.println("\nBeginning ebqlEquityHome.Client...\n");

    try {
      client = new Client(url);
    }
    catch (NamingException ne) {
      log("Unable to look up the beans home: " + ne.getMessage());
      System.exit(1);
    }

    try {
      client.runExample();
    } catch (Exception e) {
      log("There was an exception while creating and using the Equity.");
      log("This indicates that there was a problem communicating with the server: "+e);
    }

    System.out.println("\nEnd Equity.Client...\n");
  }

}
