package tests.homeMethod;

import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.homeMethod.EquityHome;

public class Client {
	private EquityHome home;
	private String url;

	public Client(String url) throws NamingException {
		this.url = url;
		home = lookupHome();
	}

	private EquityHome lookupHome() throws NamingException {
		Context ctx = getInitialContext();

		try {
			log("Looking up Home:homeMethodEquityBean");
			Object home = (EquityHome) ctx.lookup("homeMethodEquityBean");
			return (EquityHome) PortableRemoteObject.narrow(home, EquityHome.class);

		}
		catch (NamingException ne) {
			log(
				"The client was unable to lookup the EJBHome.  Please make sure "
					+ "that you have deployed the ejb with the JNDI name "
					+ "homeMethodEquityBean on the WebLogic server at "
					+ url);
			throw ne;
		}
	}

	public void runExample() {

		log("Starting the example...\n");

		try {
			int count = 0;
			log("Finding Count of Symbols Using An EquityHome Object");
			count = home.getCountOfSymbols();
			log("Current Count of Symbols:" + count);
		}
		catch (Exception e) {
			log("Error Counting Equities: " + e.getMessage());
			System.exit(1);
		}

		log("End of the example...\n");
	}

	private Context getInitialContext() throws NamingException {

		try {
			// Get an InitialContext
			Properties props = new Properties();
			props.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
			props.put(Context.PROVIDER_URL, url);
			return new InitialContext(props);
		}
		catch (NamingException ne) {
			log("We were unable to get a connection to the WebLogic server at " + url);
			log("Please make sure that the server is running.");
			throw ne;
		}
	}

	private static void log(String s) {
		System.out.println(s);
	}

	public static void main(String[] args) {
		String url = "t3://localhost:7001";
		Client client = null;

		System.out.println("\nBeginning equity.Client...\n");

		try {
			client = new Client(url);
		}
		catch (NamingException ne) {
			log("Unable to look up the beans home: " + ne.getMessage());
			System.exit(1);
		}

		try {
			client.runExample();
		}
		catch (Exception e) {
			log("There was an exception while creating and using the Equity.");
			log("This indicates that there was a problem communicating with the server: " + e);
		}

		System.out.println("\nEnd Equity.Client...\n");
	}

}
