package tests.notifications;

import java.util.Properties;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

public class DebugSubscriber implements MessageListener
{
   private TopicConnection             topicConnection=null;
   private TopicSession                topicSession=null;
   private TopicSubscriber             topicSubscriber=null;
   private Topic                       topic=null;
   private TopicConnectionFactory      topicFactory=null;
   private int                         count_=0;
   private Context                     context=null;
   private String                      url="t3://localhost:7001";

   public DebugSubscriber() throws JMSException, NamingException
   { 
     String            factoryJNDI="BookJMSFactory";
     String            topicJNDI="BookJMSTopic";

     // Get the initial context
     System.out.println("Getting Initial Context:");
     context = getInitialContext();
     System.out.println("Got Initial Context:"+context);

     // Get the connection factory
     System.out.println("Getting Topic Factory:");
     topicFactory=(TopicConnectionFactory)context.lookup(factoryJNDI);
     System.out.println("Got Topic Factory:"+topicFactory);
      
     // Create the connection 
     topicConnection = topicFactory.createTopicConnection();

     // Create the session
     topicSession=topicConnection.createTopicSession(false,Session.AUTO_ACKNOWLEDGE);
      
     // Look up the destination
     topic = (Topic)context.lookup(topicJNDI);
      
     // Create a subsriber
     topicSubscriber = topicSession.createSubscriber(topic);

     // Set the message listener, which is this class since we implement
     // the MessageListener interface
     topicSubscriber.setMessageListener(this);
   
     System.out.println("DeviceSubscriber subscribed to topic: " + topicJNDI);
     // OBS! For the message listener to receive any messages the connection
     // has to be started
     topicConnection.start();
   }
    
   public void onMessage(Message m) {

     try {
       String msg = ((TextMessage)m).getText();
       System.out.println("DeviceSubscriber got message: " + msg);
     }
     catch(Exception ex) {
       System.err.println("Device Could not handle message: " + ex);
       ex.printStackTrace();
     }
   }
    
   public void close() throws JMSException {
      topicSession.close();
      topicConnection.close();
   }

  private Context getInitialContext() throws NamingException {

    try {
      // Get an InitialContext
      Properties props = new Properties();
      props.put(Context.INITIAL_CONTEXT_FACTORY,"weblogic.jndi.WLInitialContextFactory");
      props.put(Context.PROVIDER_URL, url);
      return new InitialContext(props);
    } catch (NamingException ne) {
      log("We were unable to get a connection to the WebLogic server at "+url);
      log("Please make sure that the server is running.");
      throw ne;
    }
  }


  private void log(String str){
    System.out.println("JMSSubscriber::"+str);
  }
   
   public static void main(String[] args){
     DebugSubscriber              subscriber=null;

     try{
       System.out.println("Starting Debugging Subscriber");
       subscriber=new DebugSubscriber();
       while(true){
         Thread.sleep(10000);
       }
     }
     catch(Exception e){
       System.out.println("Error Starting Device DebugClient");
       e.printStackTrace();
     }
   }

}

