package tests.one2onerelation;
//////////////////////////////////////////////////////////////////////////////
/*
  File: Client.java

  Date          Author                                  Changes
  ---------------------------------------------------------------------------
  30 Arp 2002    Mark B. Whipple                        Created
*/
//////////////////////////////////////////////////////////////////////////////
import java.util.Properties;

import javax.rmi.PortableRemoteObject;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;
import ejbs.one2onerelation.*;


//////////////////////////////////////////////////////////////////////////
/**
  <P>
  <PRE>
  <B>Source File:             </B>Client.java
  <B>Repository Identifier:   </B>Client.java:1.0
  </P>

  <P>
  <B>Description:</B>
  </P>


  <B>IDL Definition:</B>
  </PRE>
  @author   Mark B. Whipple
  @version  1.0
  @see
**/
//////////////////////////////////////////////////////////////////////////
public class Client {
  private String url;
  private HelperHome home;

//////////////////////////////////////////////////////////////////////////
/**
  <P>
  <B>Description:</B>
  </P>
**/
//////////////////////////////////////////////////////////////////////////
  public Client(String url)throws NamingException {

    this.url=url;
    home = lookupHome();
  }

//////////////////////////////////////////////////////////////////////////
/**
  <P>
  <B>Description:</B>
  </P>
**/
//////////////////////////////////////////////////////////////////////////
  private HelperHome lookupHome() throws NamingException{
    Context ctx = getInitialContext();

    try {
      log("Looking up Home:helperHome");
      Object home = (HelperHome) ctx.lookup("one2oneRelationHome");
      return (HelperHome) PortableRemoteObject.narrow(home, HelperHome.class);

    }
    catch (NamingException ne) {
      log("The client was unable to lookup the HelperHome.  Please make sure " +
      "that you have deployed the ejb with the JNDI name " + 
      "HelperHome on the WebLogic server at "+url);
      throw ne;
    }
  }

//////////////////////////////////////////////////////////////////////////
/**
  <P>
  <B>Description:</B>
  </P>
**/
//////////////////////////////////////////////////////////////////////////
   public void runExample() {
     Helper                         helper=null;

     log("Starting the example...\n");

     try {
       log("Creating A Helper");
       helper=(Helper)PortableRemoteObject.narrow(home.create(), Helper.class);
       log("\nCreate Addresses and Owners:");
       helper.createAddressesAndOwners();

       log("\nPrint Owners and Addresses");
       helper.printOwners();

       int i = 0;
       log("\nCascade Delete:");
       log("Removing Owner " + i + " and cascade delete its addresses");
       helper.removeOwner_CascadeDeleteAddresses(i);
       log("\nPrint Owners and Addresses");
       helper.printOwners();

       log("\nRemove All Customers...");
       helper.removeAllCustomers();
     }
     catch(Exception e) {
       log("Error creating Helper: " + e.getMessage());
       System.exit(1);
     }

     log("End Part A of the example...\n");
   }



//////////////////////////////////////////////////////////////////////////
/**
  <P>
  <B>Description:</B>
  </P>
**/
//////////////////////////////////////////////////////////////////////////
  private Context getInitialContext() throws NamingException {

    try {
      // Get an InitialContext
      Properties props = new Properties();
      props.put(Context.INITIAL_CONTEXT_FACTORY,"weblogic.jndi.WLInitialContextFactory");
      props.put(Context.PROVIDER_URL, url);
      return new InitialContext(props);
    } catch (NamingException ne) {
      log("We were unable to get a connection to the WebLogic server at "+url);
      log("Please make sure that the server is running.");
      throw ne;
    }
  }

//////////////////////////////////////////////////////////////////////////
/**
  <P>
  <B>Description:</B>
  </P>
**/
//////////////////////////////////////////////////////////////////////////
  private static void log(String s) {
    System.out.println(s);
  }

//////////////////////////////////////////////////////////////////////////
/**
  <P>
  <B>Description:</B>
  </P>
**/
//////////////////////////////////////////////////////////////////////////
  public static void main(String[] args) {
    String                        url="t3://localhost:7001";
    Client                        client = null;

    System.out.println("\nBeginning cascadeDelete.Client...\n");


    try {
      client = new Client(url);
    }
    catch (NamingException ne) {
      log("Unable to look up the beans home: " + ne.getMessage());
      System.exit(1);
    }

    try {
      client.runExample();
    }
    catch (Exception e) {
      log("There was an exception while creating and using the Helper.");
      log("This indicates that there was a problem communicating with the server: "+e);
    }

    System.out.println("\nEnd cascadeDelete.Client...\n");
  }

}
