package tests.sessionFacade;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.sessionFacade.Helper;
import ejbs.sessionFacade.HelperHome;


public class Client {
  private String url;
  private HelperHome home;

  public Client(String url)throws NamingException {

    this.url=url;
    home = lookupHome();
  }

  private HelperHome lookupHome() throws NamingException{
    Context ctx = getInitialContext();

    try {
      log("Looking up Home:sessionFacade.HelperHome");
      Object home = (HelperHome) ctx.lookup("sessionFacade.HelperHome");
      return (HelperHome) PortableRemoteObject.narrow(home, HelperHome.class);

    }
    catch (NamingException ne) {
      log("The client was unable to lookup the sessionFacade.HelperHome.  Please make sure " +
      "that you have deployed the ejb with the JNDI name " + 
      "sessionFacade.HelperHome on the WebLogic server at "+url);
      throw ne;
    }
  }

   public void runExample() {
     Helper                         helper=null;

     log("Starting the example...\n");

     try {
       log("Creating A Helper");
       helper=(Helper)PortableRemoteObject.narrow(home.create(), Helper.class);
       log("\nCreate User List:");
       helper.createUserList();

       boolean isLoggedIn=helper.login("user_1","passwd_1");
       log("User: user_1 with Password: passwd_1 logged in OK:"+isLoggedIn);
       isLoggedIn=helper.login("user_1","password_1");
       log("User: user_1 with Password: password_1 logged in OK:"+isLoggedIn);

       helper.removeAll();
     }
     catch(Exception e) {
       log("Error creating Helper: " + e.getMessage());
       System.exit(1);
     }

     log("End of the example...\n");
   }



  private Context getInitialContext() throws NamingException {

    try {
      // Get an InitialContext
      Properties props = new Properties();
      props.put(Context.INITIAL_CONTEXT_FACTORY,"weblogic.jndi.WLInitialContextFactory");
      props.put(Context.PROVIDER_URL, url);
      return new InitialContext(props);
    } catch (NamingException ne) {
      log("We were unable to get a connection to the WebLogic server at "+url);
      log("Please make sure that the server is running.");
      throw ne;
    }
  }

  private static void log(String s) {
    System.out.println(s);
  }

  public static void main(String[] args) {
    String                        url="t3://localhost:7001";
    Client                        client = null;

    System.out.println("\nBeginning cascadeDelete.Client...\n");


    try {
      client = new Client(url);
    }
    catch (NamingException ne) {
      log("Unable to look up the beans home: " + ne.getMessage());
      System.exit(1);
    }

    try {
      client.runExample();
    }
    catch (Exception e) {
      log("There was an exception while creating and using the Helper.");
      log("This indicates that there was a problem communicating with the server: "+e);
    }

    System.out.println("\nEnd cascadeDelete.Client...\n");
  }

}
