package tests.userspecificdb;
import java.rmi.RemoteException;
import java.util.*;

import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.userspecificdb.UserSpecificDB;
import ejbs.userspecificdb.UserSpecificDBHome;

public class Client {
	private String url;
	private String userName;
	private String passwd;
	private UserSpecificDBHome home;

	public Client(String url, String userName, String passwd) throws NamingException {
		this.url = url;
		this.userName = userName;
		this.passwd = passwd;

		home = lookupHome();
	}

	public void example() throws CreateException, RemoteException, RemoveException {
		Vector vector = null;

		log("Creating a UserSpecificDB Bean");
		UserSpecificDB bean = (UserSpecificDB) narrow(home.create(), UserSpecificDB.class);

		vector = bean.getAllSymbols();

		Enumeration enum = vector.elements();
		while (enum.hasMoreElements()) {
			System.out.println("Found Equity:" + enum.nextElement());
		}

	}

	private Object narrow(Object ref, Class c) {
		return PortableRemoteObject.narrow(ref, c);
	}

	private UserSpecificDBHome lookupHome() throws NamingException {
		Hashtable env = new Hashtable();

		env.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
		env.put(Context.PROVIDER_URL, "t3://localhost:7001");
		if ((userName != null) && (passwd != null)) {
			env.put(Context.SECURITY_AUTHENTICATION, "simple");
			env.put(Context.SECURITY_PRINCIPAL, userName);
			env.put(Context.SECURITY_CREDENTIALS, passwd);
		}

		Context ctx = new InitialContext(env);

		log("Looking up Home object:");

		try {
			Object home = ctx.lookup("userSpecificDBHome");
			return (UserSpecificDBHome) narrow(home, UserSpecificDBHome.class);
		}
		catch (NamingException ne) {
			log("The client was unable to lookup the UserSpecificDBHome.  Please make sure ");
			log("that you have deployed the ejb with the JNDI name userSpecificDB on the WebLogic server at " + url);
			throw ne;
		}
	}

	private static void log(String s) {
		System.out.println(s);
	}

	public static void main(String[] args) throws Exception {
		String userName = "markw";
		String passwd = "passwd";
		Client client = null;

		log("\nBeginning statelessSession.Client...\n");

		if (args.length > 1) {
			userName = args[0];
			passwd = args[1];
		}

		log("\n\nTesting with User:guest");
		try {
			client = new Client("t3://localhost:7001", null, null);
		}
		catch (NamingException ne) {
			ne.printStackTrace();
			System.exit(1);
		}

		try {
			client.example();
		}
		catch (Exception e) {
			log("There was an exception while creating and using the UserSpecificDB.");
			log("This indicates that there was a problem communicating with the server: " + e);
		}

		log("\n\nTesting with User:" + userName + " and password:" + passwd + "...\n");
		try {
			client = new Client("t3://localhost:7001", userName, passwd);
		}
		catch (NamingException ne) {
			ne.printStackTrace();
			System.exit(1);
		}

		try {
			client.example();
		}
		catch (Exception e) {
			log("There was an exception while creating and using the UserSpecificDB.");
			log("This indicates that there was a problem communicating with the server: " + e);
		}

		log("\nEnd statelessSession.Client...\n");
	}

}
