package ejbs.asyncSession;

import java.util.Hashtable;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.jms.MapMessage;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

public class AsyncProcessBean implements SessionBean {

	private SessionContext ctx;
	private Hashtable params = new Hashtable();

	private void log(String s) {
		System.out.println("AsyncProcessBean::" + s);
	}

	public void ejbActivate() {
		log("ejbActivate called");
	}

	public void ejbRemove() {
		log("ejbRemove called");
	}

	public void ejbPassivate() {
		log("ejbPassivate called");
	}

	public void setSessionContext(SessionContext ctx) {
		log("setSessionContext called");
		this.ctx = ctx;
	}

	public void ejbCreate() throws CreateException {
		log("ejbCreate called");
	}

	public String startProcess() {
		String processId = null;

		log("------[ Starting Process ]---------");
		processId = "" + System.currentTimeMillis();
		log("Process ID:" + processId);
		sendMessage(processId);
		createStatus(processId);
		log("------[ Starting Process ]---------");

		return processId;
	}

	public void createStatus(String processId) {
		int status = 0;
		Context ctx = null;
		StatusHome home = null;

		try {
			ctx = new InitialContext();
			log("Looking up Home:statusHome");
			home = (StatusHome) ctx.lookup("statusHome");
			home = (StatusHome) PortableRemoteObject.narrow(home, StatusHome.class);
			home.create(processId, status);
		}
		catch (Exception e) {
			e.printStackTrace();
			log(
				"The client was unable to lookup the EJBHome.  Please make sure "
					+ "that you have deployed the ejb with the JNDI name "
					+ "statusHome on the WebLogic server");
		}
	}

	public int getStatus(String processId) {
		int status = 0;
		Context ctx = null;
		StatusHome home = null;
		Status statusBean = null;

		try {
			ctx = new InitialContext();
			log("Looking up Home:statusHome");
			home = (StatusHome) ctx.lookup("statusHome");
			home = (StatusHome) PortableRemoteObject.narrow(home, StatusHome.class);
			statusBean = (Status) PortableRemoteObject.narrow(home.findByPrimaryKey(processId), Status.class);
			status = statusBean.getStatus();
			ctx.close();
		}
		catch (Exception e) {
			e.printStackTrace();
			log(
				"The client was unable to lookup the EJBHome.  Please make sure "
					+ "that you have deployed the ejb with the JNDI name "
					+ "statusHome on the WebLogic server");
		}

		return status;
	}

	private void sendMessage(String processId) {
		TopicConnection topicConnection = null;
		TopicSession topicSession = null;
		TopicPublisher topicPublisher = null;
		Topic topic = null;
		TopicConnectionFactory topicFactory = null;

		try {
			Context context = new InitialContext();
			topicFactory = (TopicConnectionFactory) context.lookup("BookJMSFactory");
			topicConnection = topicFactory.createTopicConnection();
			topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
			topic = (Topic) context.lookup("BookJMSTopic");
			topicPublisher = topicSession.createPublisher(topic);

			MapMessage message = topicSession.createMapMessage();
			message.setString("ProcessId", processId);
			topicPublisher.publish(message);

		}
		catch (Exception e) {
			e.printStackTrace();
			log("Error Publishing Message");
		}
	}

}
