package ejbs.fileAccess;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;

import weblogic.common.T3ServicesDef;
import weblogic.io.common.T3File;
import weblogic.io.common.T3FileSystem;

public class WeblogicFileUtil implements FileUtilInterface {
  private String                    url="t3://localhost:7001";
  private String                    fileSystemName="BookFileSystem";

  

  private Context getInitialContext() {
    Context                  ctx = null;
    Properties               props = new Properties();

    try {
      props.put(Context.PROVIDER_URL, url);
      props.put(Context.INITIAL_CONTEXT_FACTORY, 
                weblogic.jndi.WLInitialContextFactory.class.getName());
      ctx = new InitialContext(props);
    }
    catch(Exception e){
      e.printStackTrace();
    }

    return ctx;
  }


  private T3File getFile(String filename) throws IOException{
    Context                  ctx = getInitialContext();
    T3File                   file = null;
    T3ServicesDef            t3s = null;
    T3FileSystem             fileSystem = null;

    try {
      t3s = (T3ServicesDef)ctx.lookup("weblogic.common.T3Services");
      fileSystem = t3s.io().getFileSystem(fileSystemName);
      file = fileSystem.getFile(filename); 
    }
    catch (Exception e) {
      log("Error Getting File from Container:"+filename);
      e.printStackTrace();
      throw new IOException("Error Getting File from Container:"+filename+"  "+e.getMessage());
    }

    return file;
  }


  public OutputStream getFileOutputStream(String filename) throws IOException{
    OutputStream             os = null;

    try {
      T3File file = getFile(filename); 
      
      if (file.exists()) {
	throw new IOException("The file already exists...");
      }
      else {
	os = file.getFileOutputStream();
      }

    }
    catch (Exception e) {
      log("Error Opening File for output:"+filename);
      e.printStackTrace();
      throw new IOException("Error Opening File for Output:"+filename+"  "+e.getMessage());
    }

    return os;
  }



  public void delete(String filename)throws IOException {
    try {
      T3File file = getFile(filename); 
      file.delete();
    }
    catch (Exception e) {
      throw new IOException("Error Deleting File:"+filename);
    }
  }


  public InputStream getFileInputStream(String filename)throws IOException {
    InputStream                    istream = null;

    try {
      T3File file = getFile(filename); 
      
      if (!file.exists()) {
	log("The file doesn't exist");
        throw new IOException("Error Opening File for Input:"+filename);
      }
      else {
      istream = file.getFileInputStream();
      }
      
    }
    catch (Exception e) {
      throw new IOException("Error Opening File for Input:"+filename+"  "+e.getMessage());
    }

    return istream;
  }

  

  private static void log(String s) {
    System.out.println("FileAccess::"+s);
  }


  public static void main(String[] argv) {
    String                 data = new String("This will be written to the file");
    byte[]                 bytes = null;
    OutputStream           ostream=null;
    InputStream            istream=null;
    WeblogicFileUtil                 client=new WeblogicFileUtil();

    try{
      log("Creating File:");
      ostream=client.getFileOutputStream("Test.txt");
      bytes=data.getBytes();
      ostream.write(bytes);
      ostream.close();

      log("Reading File:");
      istream=client.getFileInputStream("Test.txt");
      istream.read(bytes);
      istream.close();
      String result = new String(bytes);
      log("File contents: " + result);      

      log("Deleting File:");
      client.delete("Test.txt");
    }
    catch(Exception e){
      e.printStackTrace();
    }

  }

}







