package ejbs.toString;

import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

abstract public class EquityBean implements EntityBean {

	final static boolean VERBOSE = true;
	private EntityContext ctx;

	public EquityBean() {
	};

	public void setEntityContext(EntityContext ctx) {
		log("setEntityContext called (" + id() + ")");
		this.ctx = ctx;
	}

	public void unsetEntityContext() {
		log("EquityBean.unsetEntityContext (" + id() + ")");
		this.ctx = null;
	}

	abstract public String getSymbol();
	abstract public void setSymbol(String symbol);

	abstract public String getDescription();
	abstract public void setDescription(String description);

	abstract public double getLastTrade();
	abstract public void setLastTrade(double lastTrade);

	abstract public double getChange();
	abstract public void setChange(double change);

	abstract public int getVolume();
	abstract public void setVolume(int volume);

	abstract public double getMarketCap();
	abstract public void setMarketCap(double marketCap);

	abstract public double getPe();
	abstract public void setPe(double pe);

	abstract public int getAvgVolume();
	abstract public void setAvgVolume(int avgVolume);

	/**
	 * Returns the Primary Key identifying this EJBean.
	 * 
	 * @return                  String Identification
	 */
	private String id() {
		return ""
			+ System.identityHashCode(this)
			+ ", PK = "
			+ (String) ((ctx == null)
				? "nullctx"
				: ((ctx.getPrimaryKey() == null ? "null" : ctx.getPrimaryKey().toString())));
	}

	public void ejbActivate() {
		log("EquityBean.ejbActivate (" + id() + ")");
	}

	public void ejbPassivate() {
		log("EquityBean.ejbPassivate (" + id() + ")");
	}

	public void ejbLoad() {
		log("EquityBean.ejbLoad (" + id() + ")");
	}

	public void ejbStore() {
		log("EquityBean.ejbStore (" + id() + ")");
	}

	public void ejbRemove() throws RemoveException {
		log("EquityBean.ejbRemove (" + id() + ")");
	}

	public String ejbCreate(String symbol, String description) throws CreateException {
		log(
			"EquityBean.ejbCreate( id = "
				+ System.identityHashCode(this)
				+ ", PK = "
				+ symbol
				+ ", "
				+ "Description =  "
				+ description
				+ ")");

		setSymbol(symbol);
		setDescription(description);

		return null; // See 9.4.2 of the EJB 1.1 specification
	}

	public void ejbPostCreate(String symbol, String description) {
		log("EquityBean.ejbPostCreate (" + id() + ")");
	}

	// Application defined methods

	public void log(String logstr) {
		System.out.println(logstr);
	}

	public void setEquity(EquityVO equity) throws RemoteException {
		//setSymbol(equity.symbol);
		setDescription(equity.description);
		setLastTrade(equity.lastTrade);
		setChange(equity.change);
		setVolume(equity.volume);
		setMarketCap(equity.marketCap);
		setPe(equity.PE);
		setAvgVolume(equity.avgVolume);
	}

	public EquityVO getEquity() throws RemoteException {
		EquityVO equity = new EquityVO();

		equity.symbol = getSymbol();
		equity.description = getDescription();
		equity.lastTrade = getLastTrade();
		equity.change = getChange();
		equity.volume = getVolume();
		equity.marketCap = getMarketCap();
		equity.PE = getPe();
		equity.avgVolume = getAvgVolume();

		return equity;
	}

	//-----------------------[ Listing 4.2 ]---------------------//
	public String ejbToString() {
		StringBuffer buffer = new StringBuffer();
		EquityVO equity = new EquityVO();

		equity.symbol = getSymbol();
		equity.description = getDescription();
		equity.lastTrade = getLastTrade();
		equity.change = getChange();
		equity.volume = getVolume();
		equity.marketCap = getMarketCap();
		equity.PE = getPe();
		equity.avgVolume = getAvgVolume();

		buffer.append("---------------[ Equity Entity Bean ]--------------\n");
		buffer.append("Entity ID:" + id() + "\n\n");
		buffer.append(equity.toString() + "\n");
		buffer.append("---------------[ Equity Entity Bean ]--------------");

		return buffer.toString();
	}
	//-----------------------[ Listing 4.2 ]---------------------//

}
