package tests.asyncSession;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.asyncSession.AsyncProcess;
import ejbs.asyncSession.AsyncProcessHome;

public class Client {

	private AsyncProcessHome lookupHome() throws NamingException {
		Context ctx = getInitialContext();

		try {
			log("Looking up Home:asyncSessionHome");
			Object home = (AsyncProcessHome) ctx.lookup("asyncSessionHome");
			return (AsyncProcessHome) PortableRemoteObject.narrow(home, AsyncProcessHome.class);

		}
		catch (NamingException ne) {
			log(
				"The client was unable to lookup asyncSessionHome.  Please make sure "
					+ "that you have deployed the ejb with the JNDI name "
					+ "asyncSessionHome on the WebLogic server");
			throw ne;
		}
	}

	private Context getInitialContext() throws NamingException {

		try {
			// Get an InitialContext
			Properties props = new Properties();
			props.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
			props.put(Context.PROVIDER_URL, "t3://localhost:7001");
			return new InitialContext(props);
		}
		catch (NamingException ne) {
			log("We were unable to get a connection to the WebLogic server");
			log("Please make sure that the server is running.");
			throw ne;
		}
	}

	public void runExample() {
		AsyncProcess helper = null;
		AsyncProcessHome home = null;
		String processId = null;
		String processId2 = null;
		int status = -1;
		int status2 = -1;

		try {
			log("Creating A AsyncProcess Helper");
			home = lookupHome();
			helper = (AsyncProcess) PortableRemoteObject.narrow(home.create(), AsyncProcess.class);
			processId = helper.startProcess();
			log("Process Id for newly started Async Process:" + processId);

			processId2 = helper.startProcess();
			log("Process Id2 for newly started Async Process:" + processId);

			while ((status != 1) && (status2 != 1)) {
				status = helper.getStatus(processId);
				log("Status for Process Id:" + processId + "=" + status);

				status2 = helper.getStatus(processId2);
				log("Status for Process Id2:" + processId2 + "=" + status2);

				try {
					log("Sleeping.....");
					Thread.sleep(2000);
				}
				catch (InterruptedException ie) {
				}
			}

		}
		catch (Exception e) {
			log("Error Handling file in Bean: ");
			e.printStackTrace();
		}

	}

	public static void log(String s) {
		System.out.println("Init Params Client::" + s);
	}

	public static void main(String[] argv) {
		Client client = new Client();

		client.runExample();
	}

}
