package tests.emailBean;

import java.rmi.RemoteException;
import java.util.Properties;

import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.emailBean.Emailer;
import ejbs.emailBean.EmailerHome;

public class Client {

	private static final String JNDI_NAME = "emailerBean";

	private String url = "t3://localhost:7001";
	private EmailerHome home;

	public Client() throws NamingException {

		home = lookupHome();
	}

	public void runExample() throws CreateException, RemoteException, RemoveException {
		Emailer emailer = null;

		log("Creating an emailer Session Bean");
		emailer = (Emailer) narrow(home.create(), Emailer.class);

		emailer.sendEmail("mbwhipple@yahoo.com", "This is a test message");
		log("Removing the emailer");
		emailer.remove();

	}

	private Object narrow(Object ref, Class c) {
		return PortableRemoteObject.narrow(ref, c);
	}

	private EmailerHome lookupHome() throws NamingException {
		// Lookup the beans home using JNDI
		Context ctx = getInitialContext();

		try {
			Object home = ctx.lookup(JNDI_NAME);
			return (EmailerHome) narrow(home, EmailerHome.class);
		}
		catch (NamingException ne) {
			log("The client was unable to lookup the EJBHome.  Please make sure ");
			log("that you have deployed the ejb with the JNDI name " + JNDI_NAME + " on the WebLogic server at " + url);
			throw ne;
		}
	}

	private Context getInitialContext() throws NamingException {

		try {
			// Get an InitialContext
			Properties h = new Properties();
			h.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
			h.put(Context.PROVIDER_URL, url);
			return new InitialContext(h);
		}
		catch (NamingException ne) {
			log("We were unable to get a connection to the WebLogic server at " + url);
			log("Please make sure that the server is running.");
			throw ne;
		}
	}

	private static void log(String s) {
		System.out.println(s);
	}

	public static void main(String[] args) throws Exception {
		String url = "t3://localhost:7001";
		Client client = null;

		log("\nBeginning tradesession.Client...\n");

		try {
			client = new Client();
			client.runExample();
		}
		catch (Exception e) {
			log("There was an exception while creating and using the Emailer.");
			log("This indicates that there was a problem communicating with the server: " + e);
		}

		log("\nEnd statelessSession.Client...\n");
	}

}
