package tests.fileAccess;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.fileAccess.FileAccess;
import ejbs.fileAccess.FileAccessHome;
import ejbs.fileAccess.FileUtilInterface;
import ejbs.fileAccess.WeblogicFileUtil;

public class Client {

	private FileAccessHome lookupHome() throws NamingException {
		Context ctx = getInitialContext();

		try {
			log("Looking up Home:fileAccessHome");
			Object home = (FileAccessHome) ctx.lookup("fileAccessHome");
			return (FileAccessHome) PortableRemoteObject.narrow(home, FileAccessHome.class);

		}
		catch (NamingException ne) {
			log(
				"The client was unable to lookup fileAccessHome.  Please make sure "
					+ "that you have deployed the ejb with the JNDI name "
					+ "fileAccessHome on the WebLogic server");
			throw ne;
		}
	}

	private Context getInitialContext() throws NamingException {

		try {
			// Get an InitialContext
			Properties props = new Properties();
			props.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
			props.put(Context.PROVIDER_URL, "t3://localhost:7001");
			return new InitialContext(props);
		}
		catch (NamingException ne) {
			log("We were unable to get a connection to the WebLogic server");
			log("Please make sure that the server is running.");
			throw ne;
		}
	}

	public String beanFileHandling() {
		FileAccess helper = null;
		String data = null;
		String filename = "TextFromBean.txt";
		FileAccessHome home = null;

		try {
			log("Creating A FileAccess Helper");
			home = lookupHome();
			helper = (FileAccess) PortableRemoteObject.narrow(home.create(), FileAccess.class);
			helper.createFile(filename, "This file is created from a bean");
			data = helper.readFile(filename);
			helper.deleteFile(filename);
		}
		catch (Exception e) {
			log("Error Handling file in Bean: ");
			e.printStackTrace();
		}

		return data;
	}

	public void remoteFileHandling() {
		String data = new String("This will be written to the file");
		byte[] bytes = new byte[1024];
		int bytesRead = 0;
		StringBuffer buffer = new StringBuffer();
		OutputStream ostream = null;
		InputStream istream = null;
		FileUtilInterface client = new WeblogicFileUtil();

		try {
			System.out.println("Creating File:");
			ostream = client.getFileOutputStream("Test.txt");
			ostream.write(data.getBytes());
			ostream.close();

			System.out.println("Reading File:");
			istream = client.getFileInputStream("Test.txt");
			while ((bytesRead = istream.read(bytes)) > 0) {
				buffer.append(new String(bytes, 0, bytesRead));
			}
			istream.close();
			System.out.println("File contents: " + buffer.toString());

			System.out.println("Deleting File:");
			client.delete("Test.txt");
		}
		catch (Exception e) {
			e.printStackTrace();
		}

	}

	public static void log(String s) {
		System.out.println("File Handling Client::" + s);
	}

	public static void main(String[] argv) {
		Client client = new Client();

		log("---------[ Using Remote File Handling ]---------");
		client.remoteFileHandling();
		log("---------[ Using Remote File Handling ]---------\n\n");

		log("---------[ Using EJB File Handling ]---------");
		log("File Contents:" + client.beanFileHandling());
		log("---------[ Using EJB File Handling ]---------");
	}

}
