package tests.methodRedux;
import java.util.Properties;

import javax.jms.MapMessage;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import ejbs.methodRedux.TestEntity;
import ejbs.methodRedux.TestEntityHome;
import ejbs.methodRedux.TestSession;
import ejbs.methodRedux.TestSessionHome;

public class Client {

	private TestEntityHome lookupEntityHome() throws NamingException {
		Context ctx = getInitialContext();

		try {
			log("Looking up Home:testEntityHome");
			Object home = (TestEntityHome) ctx.lookup("testEntityHome");
			return (TestEntityHome) PortableRemoteObject.narrow(home, TestEntityHome.class);

		}
		catch (NamingException ne) {
			log(
				"The client was unable to lookup testEntityHome.  Please make sure "
					+ "that you have deployed the ejb with the JNDI name "
					+ "testEntityHome on the WebLogic server");
			throw ne;
		}
	}

	private TestSessionHome lookupSessionHome() throws NamingException {
		Context ctx = getInitialContext();

		try {
			log("Looking up Home:asyncSessionHome");
			Object home = (TestSessionHome) ctx.lookup("testSessionHome");
			return (TestSessionHome) PortableRemoteObject.narrow(home, TestSessionHome.class);

		}
		catch (NamingException ne) {
			log(
				"The client was unable to lookup asyncSessionHome.  Please make sure "
					+ "that you have deployed the ejb with the JNDI name "
					+ "asyncSessionHome on the WebLogic server");
			throw ne;
		}
	}

	private Context getInitialContext() throws NamingException {

		try {
			// Get an InitialContext
			Properties props = new Properties();
			props.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
			props.put(Context.PROVIDER_URL, "t3://localhost:7001");
			return new InitialContext(props);
		}
		catch (NamingException ne) {
			log("We were unable to get a connection to the WebLogic server");
			log("Please make sure that the server is running.");
			throw ne;
		}
	}

	public void runExample() {
		TestSession session = null;
		TestEntity entity = null;

		try {

			log("Creating A TestSession Bean");
			TestSessionHome sessionHome = lookupSessionHome();
			session = (TestSession) PortableRemoteObject.narrow(sessionHome.create(), TestSession.class);
			session.doFunction("Hello World");

			log("Creating A TestEntity Bean");
			TestEntityHome entityHome = lookupEntityHome();
			entity = (TestEntity) PortableRemoteObject.narrow(entityHome.create("1234", 1), TestEntity.class);
			entity.remove();

			sendMessage("Hello World");

		}
		catch (Exception e) {
			log("Error Handling file in Bean: ");
			e.printStackTrace();
		}

	}

	private void sendMessage(String message) {
		TopicConnection topicConnection = null;
		TopicSession topicSession = null;
		TopicPublisher topicPublisher = null;
		Topic topic = null;
		TopicConnectionFactory topicFactory = null;

		try {
			Context context = getInitialContext();
			topicFactory = (TopicConnectionFactory) context.lookup("BookJMSFactory");
			topicConnection = topicFactory.createTopicConnection();
			topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
			topic = (Topic) context.lookup("BookJMSTopic");
			topicPublisher = topicSession.createPublisher(topic);

			MapMessage msg = topicSession.createMapMessage();
			msg.setString("Message", message);
			topicPublisher.publish(msg);

		}
		catch (Exception e) {
			e.printStackTrace();
			log("Error Publishing Message");
		}
	}

	public static void log(String s) {
		System.out.println("Init Params Client::" + s);
	}

	public static void main(String[] argv) {
		Client client = new Client();

		client.runExample();
	}

}
