
package ejbs.errorHandling;
import java.rmi.RemoteException;
import java.util.Properties;

import javax.ejb.CreateException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

public class MessageBean implements MessageDrivenBean, MessageListener {
	private static final boolean VERBOSE = true;
	private MessageDrivenContext ctx;

	public void ejbRemove() {
		log("ejbRemove called");
	}

	public void ejbPassivate() {
		log("ejbPassivate called");
	}

	public void setMessageDrivenContext(MessageDrivenContext ctx) {
		log("setMessageDrivenContext called");
		this.ctx = ctx;
	}

	public void ejbCreate() throws CreateException {
		log("ejbCreate called - This is called by the Container when deploying bean");

	}

	public void onMessage(Message msg) {
		MapMessage map = (MapMessage) msg;
		String symbol = null;
		String description = null;

		try {
			symbol = map.getString("Symbol");
			description = map.getString("Description");

			System.out.println("Received Symbol : " + symbol);
			System.out.println("Received Description : " + description);

			processEquityMessage(symbol, description);
		}
		catch (Exception e) {
			e.printStackTrace();
			System.out.println("Error Creating Equity with Symbol:" + symbol);
			System.out.println("Consuming error and " + "passing on to error Handler");
			try {
				handleError(e, msg);
			}
			catch (Exception errorExc) {
			}
		}
	}

	private void processEquityMessage(String symbol, String description) {
		log("Processing Equity:" + symbol + " Description:" + description);
		throw new RuntimeException("There is a problem processing this equity:" + symbol);
	}

	private void handleError(Exception e, Message msg) {
		ErrorHandler handler = lookupErrorEJB();
		try {
			handler.handleMessageDrivenError(e.getMessage(), msg);
		}
		catch (RemoteException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	}

	private ErrorHandler lookupErrorEJB() {
		Context ctx = null;
		ErrorHandler handler=null;

		try {
			Properties props = new Properties();
			props.put(Context.INITIAL_CONTEXT_FACTORY, "weblogic.jndi.WLInitialContextFactory");
			props.put(Context.PROVIDER_URL, "t3://localhost:7001");
			ctx = new InitialContext(props);
			
			log("Looking up Home:errorHandlerBean");
			ErrorHandlerHome home = (ErrorHandlerHome) ctx.lookup("errorHandlerBean");
			home = (ErrorHandlerHome) PortableRemoteObject.narrow(home, ErrorHandlerHome.class);
			handler = (ErrorHandler) PortableRemoteObject.narrow(home.create(), ErrorHandler.class);
		}
		catch (Exception e) {
			e.printStackTrace();
			log("The client was unable to lookup the errorHandlerBean.  Please make sure "
					+ "that you have deployed the ejb with the JNDI name "
					+ "errorHandlerBean on the WebLogic server");
		}
		return handler;
	}

	private void log(String s) {
		System.out.println(s);
	}

	public void ejbActivate() {
		log("ejbActivate called");
	}

}
