package ejbs.rollback;

import java.util.Hashtable;

import javax.ejb.CreateException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;

public class MessageBean implements MessageDrivenBean, MessageListener {
	private MessageDrivenContext mdbContext;
	private static Hashtable previousMessages= new Hashtable();
	private int count = 0;
	private static final int _MAX_REDLIVERY_CONST_ = 3;

	public void ejbRemove() {
		log("ejbRemove called");
	}

	public void ejbPassivate() {
		log("ejbPassivate called");
	}

	public void setMessageDrivenContext(MessageDrivenContext mdbContext) {
		log("setMessageDrivenContext called");
		this.mdbContext = mdbContext;
	}

	public void ejbCreate() throws CreateException {
		log("ejbCreate called - This is called by the Container when deploying bean");

	}

	public void onMessage(Message incomingMsg) {
		// get the unique message Id 
		String msgId;
		try {
			msgId = incomingMsg.getJMSMessageID();
			log("Received Message with ID:"+msgId);

			if (previousMessages.containsKey(msgId)){
				count = ((Integer) previousMessages.get(msgId)).intValue();
			}
			else{
				count = 0;
			}

			log("Number of Times Message previously received:"+count);


			// if msg has been retried couple of times, discard it.
			//   and remove the stored id.
			if (count > _MAX_REDLIVERY_CONST_) {
				logMessage(incomingMsg);
				previousMessages.remove(msgId);
				return;
			}

			//perform business logic for message
			boolean error = doBusinessFunction();
			log("doBusinessFunction returned error status:"+error);

			if (error) {
				log("The business logic failed.  Rolling back Message Transaction");
				mdbContext.setRollbackOnly();
				previousMessages.put(msgId, new Integer(++count));

			}
			else {
				log("Business logic performed OK");
				if (previousMessages.containsKey(msgId)){
					log("Removing Message from HashMap");
					previousMessages.remove(msgId);
				}
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}

	}

	private void log(String s) {
		System.out.println("Rollback MessageBean::"+s);
	}

	private void logMessage(Message msg) {
		try {
			log("Handling Message with Message ID:" + msg.getJMSMessageID());
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	private boolean doBusinessFunction() {
		System.out.println("Executing Business function that will fail");
		return true;
	}

	public void ejbActivate() {
		log("ejbActivate called");
	}

}
