package tests.concurrent;

import java.util.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.jms.QueueConnectionFactory;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.QueueSender;
import javax.jms.Queue;
import javax.jms.MapMessage;
import javax.jms.Session;
import javax.jms.JMSException;

public class Client 
{
   private QueueConnection          queueConnection=null;
   private QueueSession             queueSession=null;
   private QueueSender           queueSender=null;
   private Queue                    queue=null;
   private QueueConnectionFactory   queueFactory = null;
   private String                   url="t3://localhost:7001";
   
  private Context getInitialContext() throws NamingException {

    try {
      // Get an InitialContext
      Properties props = new Properties();
      props.put(Context.INITIAL_CONTEXT_FACTORY,"weblogic.jndi.WLInitialContextFactory");
      props.put(Context.PROVIDER_URL, url);
      return new InitialContext(props);
    } catch (NamingException ne) {
      log("We were unable to get a connection to the WebLogic server at "+url);
      log("Please make sure that the server is running.");
      throw ne;
    }
  }



   public Client()throws JMSException, NamingException {
      Context context = getInitialContext();

      queueFactory=(QueueConnectionFactory)context.lookup("BookJMSFactory");
      queueConnection = queueFactory.createQueueConnection();
      queueSession=queueConnection.createQueueSession(false,Session.AUTO_ACKNOWLEDGE);
      queue = (Queue)context.lookup("BookJMSConcurrentQueue");
      queueSender = queueSession.createSender(queue);
   }


   public void send(String symbol,String description) throws JMSException {

     // Create a message
     MapMessage message = queueSession.createMapMessage();
     message.setString("Symbol",symbol);
     message.setString("Description",description);

     queueSender.send(message);
   }


   public void close() throws JMSException {
     queueConnection.close();
   }


   public void log(String str){
     System.out.println("Client::"+str);
   }


   public static void main(String[] args) {
     Client          sender=null;

     try{
       sender=new Client();
   
         String symbol = "SUNW";
         String description = "SUN Systems";
         System.out.println("Publishing message: "+symbol+" "+description);
         sender.send(symbol,description);

         symbol = "IBM";
         description = "International Business Mach";
         System.out.println("Publishing message: "+symbol+" "+description);
         sender.send(symbol,description);
   
       // Close down your sender
       sender.close();
   
     }
     catch(Exception ex) {
       System.err.println("An exception occurred while testing Pub: " + ex);
       ex.printStackTrace();
     }
   }
   
}
