package tests.fifo;

import java.util.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.jms.QueueConnectionFactory;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.QueueSender;
import javax.jms.Queue;
import javax.jms.MapMessage;
import javax.jms.Session;
import javax.jms.JMSException;

public class Client 
{
   private QueueConnection          queueConnection=null;
   private QueueSession             queueSession=null;
   private QueueSender              queueSender=null;
   private Queue                    queue=null;
   private QueueConnectionFactory   queueFactory = null;
   private String                   url="t3://localhost:7001";
   
  private Context getInitialContext() throws NamingException {

    try {
      // Get an InitialContext
      Properties props = new Properties();
      props.put(Context.INITIAL_CONTEXT_FACTORY,"weblogic.jndi.WLInitialContextFactory");
      props.put(Context.PROVIDER_URL, url);
      return new InitialContext(props);
    } catch (NamingException ne) {
      log("We were unable to get a connection to the WebLogic server at "+url);
      log("Please make sure that the server is running.");
      throw ne;
    }
  }


   public Client()throws JMSException, NamingException {
      Context context = getInitialContext();

      queueFactory=(QueueConnectionFactory)context.lookup("BookJMSFactory");
      queueConnection = queueFactory.createQueueConnection();
      queueSession=queueConnection.createQueueSession(false,Session.AUTO_ACKNOWLEDGE);
      queue = (Queue)context.lookup("BookJMSFifoQueue");
      queueSender = queueSession.createSender(queue);
   }

   public void send() throws JMSException {
     MapMessage message = null;

     for(int i=0;i<10;i++){
       message = queueSession.createMapMessage();
       message.setInt("Index",i);
       queueSender.send(message);
     }

   }

   public void close() throws JMSException {
     queueConnection.close();
   }

   public void log(String str){
     System.out.println("Client::"+str);
   }

   public static void main(String[] args) {
     Client          sender=null;

     try{
       sender=new Client();
       System.out.println("Publishing messages:");
       sender.send();
       sender.close();
     }
     catch(Exception ex) {
       System.err.println("An exception occurred while testing Pub: " + ex);
       ex.printStackTrace();
     }
   }
   
}
