package tests.streamMessage;

import java.util.*;
import java.io.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.jms.TopicConnectionFactory;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.jms.TopicPublisher;
import javax.jms.Topic;
import javax.jms.BytesMessage;
import javax.jms.Session;
import javax.jms.JMSException;

public class Client 
{
   private TopicConnection          topicConnection=null;
   private TopicSession             topicSession=null;
   private TopicPublisher           topicPublisher=null;
   private Topic                    topic=null;
   private TopicConnectionFactory   topicFactory = null;
   private String                   url="t3://localhost:7001";
   
  private Context getInitialContext() throws NamingException {

    try {
      // Get an InitialContext
      Properties props = new Properties();
      props.put(Context.INITIAL_CONTEXT_FACTORY,"weblogic.jndi.WLInitialContextFactory");
      props.put(Context.PROVIDER_URL, url);
      return new InitialContext(props);
    } catch (NamingException ne) {
      log("We were unable to get a connection to the WebLogic server at "+url);
      log("Please make sure that the server is running.");
      throw ne;
    }
  }


   public Client( String factoryJNDI, String topicJNDI )
                    throws JMSException, NamingException {
      // Get the initial context
      Context context = getInitialContext();

      // Get the connection factory
      topicFactory=(TopicConnectionFactory)context.lookup(factoryJNDI);
      
      // Create the connection 
      topicConnection = topicFactory.createTopicConnection();

      // Create the session
      topicSession=topicConnection.createTopicSession(false,
                                   Session.AUTO_ACKNOWLEDGE);
      
      // Look up the destination
      topic = (Topic)context.lookup(topicJNDI);
      
      // Create a publisher
      topicPublisher = topicSession.createPublisher(topic);
   }

   public void publish(String filename) throws JMSException {
     byte[]                       bytes=new byte[1024];
     FileInputStream              istream=null;
     int                          bytesRead=0;
     int                          totalBytesRead=0;

     try{
       BytesMessage message = topicSession.createBytesMessage();
       istream=new FileInputStream(filename);
       while(  (bytesRead=istream.read(bytes,0,bytes.length))>0  ){
         message.writeBytes(bytes,0,bytesRead);
         totalBytesRead+=bytesRead;
       }
       istream.close();
       log("Read Total Bytes:"+totalBytesRead);

       topicPublisher.publish(message);
     }
     catch(Exception e){
       log("Error Reading File");
       e.printStackTrace();
     }

   }

   public void close() throws JMSException {
     topicSession.close();
     topicConnection.close();
   }


   public void log(String str){
     System.out.println("Client::"+str);
   }


   public static void main(String[] args) {
     Client          publisher=null;
     String          filename="build.xml";

     try{
       publisher=new Client("BookJMSFactory","BookJMSStreamTopic");
       System.out.println("Publishing message:");
       if(args.length>0){
         filename=args[0];
       }

       publisher.publish(filename);
       publisher.close();
     }
     catch(Exception ex) {
       System.err.println("An exception occurred while testing Pub: " + ex);
       ex.printStackTrace();
     }
   }
   
}
