package tests.testEntity;

import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.ObjectNotFoundException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.cactus.ServletTestCase;

import ejbs.testEntity.TestEntity;
import ejbs.testEntity.TestEntityHome;

public class TestServlet extends ServletTestCase {

	public TestServlet(String theName) {
		super(theName);
	}


	public static Test suite() {
		return new TestSuite(TestServlet.class);
	}


	public void testCreateEntity() {

		try {
			TestEntityHome home = lookupHome();
			TestEntity entity = createEntity("This is a test", home);
			//assertEquals(true,loggedIn);
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Failed Finding Entity:" + e.getMessage());
		}

	}


	private TestEntity createEntity(String txt, TestEntityHome home)
		throws ObjectNotFoundException, CreateException, RemoteException {

		TestEntity entity = null;
		Integer id = null;

		log("Creating Entity with Text: " + txt);
		entity = (TestEntity) PortableRemoteObject.narrow(home.create(txt), TestEntity.class);
		id = entity.getEntityId();

		log("Created Entity with ID: " + id.intValue());
		return entity;
	}


	private TestEntity findEntity(Integer id, TestEntityHome home)
		throws ObjectNotFoundException, FinderException, CreateException, RemoteException {

		log("Trying to find Entity with ID: " + id);
		return (TestEntity) PortableRemoteObject.narrow(home.findByPrimaryKey(id), TestEntity.class);
	}


	public void testFindEntity() {
		Integer id = null;

		try {
			TestEntityHome home = lookupHome();

			//---------[ Create the entity ]--------------//
			TestEntity entity = createEntity("This is a test", home);
			id = entity.getEntityId();
			log("Created Entity with ID:" + id.intValue());

			//---------[ Find the entity ]--------------//
			entity = findEntity(id, home);

			//---------[ Check the id to be identical ]--------------//
			assertEquals(id, entity.getEntityId());
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Failed Finding Entity:" + e.getMessage());
		}

	}


	private TestEntityHome lookupHome() throws NamingException {
		Context ctx = new InitialContext();

		log("Looking up Home:");
		TestEntityHome home = (TestEntityHome) ctx.lookup("testEntity");
		home = (TestEntityHome) PortableRemoteObject.narrow(home, TestEntityHome.class);
		return home;
	}


	private void log(String str) {
		System.out.println("TestServlet::" + str);
	}

}
